#!/usr/bin/python
#
# compare basic wxmplot plotting to DISLIN and matplotlib,
# inspired from example at
#     https://www2.mps.mpg.de/dislin/exa_curv.html
#

# make data
import numpy as np
x  = 3.6*np.arange(101)
y1 = np.cos(np.pi*x/180)
y2 = np.sin(np.pi*x/180)

# dislin:
#   See script at:
#   https://www2.mps.mpg.de/dislin/exa_python.html#section_1

# matplotlib/pyplot:
mpl_script = """
import  matplotlib.pyplot as plt
plt.plot(x, y1, '-', color='r')
plt.plot(x, y2, '-+', color='g')
plt.title('DISLIN Comparison\nsin(x) and cos(x)')
plt.xlabel('x')
plt.ylabel('y')
plt.show()
"""

# wxmplot:
import wxmplot.interactive as wi
wi.plot(x, y1, color='red', xlabel='x', ylabel='y',
        title='DISLIN Comparison\nsin(x) and cos(x)')
wi.plot(x, y2, color='green3', marker='+')



#!/usr/bin/python
#
# WXMPlot GUI to Configure (2D) Plots
#
import os, sys
import time
from functools import partial
import wx
import wx.lib.colourselect  as csel
import wx.lib.agw.flatnotebook as flat_nb

import wx.lib.scrolledpanel as scrolled
import numpy as np
from wxutils import get_cwd
from .utils import (LabeledTextCtrl, MenuItem, SimpleText, Choice,
                    FloatSpin)
from .config import PlotConfig
from .colors import hexcolor, hex2rgb, mpl_color

try:
    import yaml
    HAS_YAML = True
except ImportError:
    HAS_YAML = False

FNB_STYLE = flat_nb.FNB_NO_X_BUTTON|flat_nb.FNB_SMART_TABS|flat_nb.FNB_NO_NAV_BUTTONS|flat_nb.FNB_NODRAG

ISPINSIZE = 65
FSPINSIZE = 65

def autopack(panel, sizer):
    panel.SetAutoLayout(True)
    panel.SetSizer(sizer)
    sizer.Fit(panel)

def ffmt(val):
    """format None or floating point as string"""
    if val is not None:
        try:
            return "%.5g" % val
        except:
            pass
    return repr(val)


def clean_texmath(txt):
    """
    clean tex math string, preserving control sequences
    (incluing \n, so also '\nu') inside $ $, while allowing
    \n and \t to be meaningful in the text string
    """
    s = "%s " % txt
    out = []
    i = 0
    while i < len(s)-1:
        if s[i] == '\\' and s[i+1] in ('n', 't'):
            if s[i+1] == 'n':
                out.append('\n')
            elif s[i+1] == 't':
                out.append('\t')
            i += 1
        elif s[i] == '$':
            j = s[i+1:].find('$')
            if j < 0:
                j = len(s)
            out.append(s[i:j+2])
            i += j+2
        else:
            out.append(s[i])
        i += 1
        if i > 5000:
            break
    return ''.join(out).strip()

class PlotConfigFrame(wx.Frame):
    """ GUI Configure Frame"""
    def __init__(self, parent=None, config=None, trace_color_callback=None):
        if config is None:
            config = PlotConfig()
        self.conf   = config
        if callable(trace_color_callback):
            self.conf.trace_color_callback = trace_color_callback

        self.parent = parent
        self.canvas = self.conf.canvas
        self.axes = self.canvas.figure.get_axes()
        self.conf.relabel()
        self.show_legend_cbs = []
        self.DrawPanel()
        mbar = wx.MenuBar()

        fmenu = wx.Menu()
        MenuItem(self, fmenu, "Save Configuration\tCtrl+S",
                 "Save Configuration",
                 self.save_config)
        MenuItem(self, fmenu, "Load Configuration\tCtrl+R",
                 "Load Configuration",
                 self.load_config)
        mbar.Append(fmenu, 'File')
        self.SetMenuBar(mbar)

    def save_config(self, evt=None, fname='wxmplot.yaml'):
        if not HAS_YAML:
            return
        file_choices = 'YAML Config File (*.yaml)|*.yaml'
        dlg = wx.FileDialog(self, message='Save plot configuration',
                            defaultDir=get_cwd(),
                            defaultFile=fname,
                            wildcard=file_choices,
                            style=wx.FD_SAVE|wx.FD_CHANGE_DIR)

        if dlg.ShowModal() == wx.ID_OK:
            conf = self.conf.get_current_config()
            ppath = os.path.abspath(dlg.GetPath())
            with open(ppath, 'w') as fh:
                fh.write("%s\n" % yaml.dump(conf))


    def load_config(self, evt=None):
        if not HAS_YAML:
            return
        file_choices = 'YAML Config File (*.yaml)|*.yaml'
        dlg = wx.FileDialog(self, message='Read plot configuration',
                            defaultDir=get_cwd(),
                            wildcard=file_choices,
                            style=wx.FD_OPEN)

        if dlg.ShowModal() == wx.ID_OK:
            conf = yaml.safe_load(open(os.path.abspath(dlg.GetPath()), 'r').read())
            self.conf.load_config(conf)


    def DrawPanel(self):
        style = wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, self.parent, -1, 'Configure Plot', style=style)
        bgcol = hex2rgb('#FEFEFE')
        panel = wx.Panel(self, -1)
        panel.SetBackgroundColour(bgcol)

        font = wx.Font(12,wx.SWISS,wx.NORMAL,wx.NORMAL,False)
        panel.SetFont(font)

        self.nb = flat_nb.FlatNotebook(panel, wx.ID_ANY, agwStyle=FNB_STYLE)

        self.nb.SetActiveTabColour((253, 253, 230))
        self.nb.SetTabAreaColour((bgcol[0]-8, bgcol[1]-8, bgcol[2]-8))
        self.nb.SetNonActiveTabTextColour((10, 10, 100))
        self.nb.SetActiveTabTextColour((100, 10, 10))
        self.nb.AddPage(self.make_linetrace_panel(parent=self.nb, font=font),
                        'Colors and Line Properties', True)
        self.nb.AddPage(self.make_range_panel(parent=self.nb, font=font),
                        'Ranges and Margins', True)
        self.nb.AddPage(self.make_text_panel(parent=self.nb, font=font),
                        'Text, Labels, Legends', True)
        self.nb.AddPage(self.make_scatter_panel(parent=self.nb, font=font),
                        'Scatterplot Settings',
                        self.conf.plot_type == 'scatter')
        for i in range(self.nb.GetPageCount()):
            self.nb.GetPage(i).SetBackgroundColour(bgcol)

        self.nb.SetSelection(0)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sty = wx.ALIGN_LEFT|wx.LEFT|wx.TOP|wx.BOTTOM|wx.EXPAND

        sizer.Add(self.nb, 1, wx.GROW|sty, 3)
        autopack(panel, sizer)
        self.SetMinSize((875, 250))
        self.SetSize((975, 450))
        self.Show()
        self.Raise()

    def make_range_panel(self, parent, font=None):
        # bounds, margins, scales
        panel = wx.Panel(parent)
        if font is None:
            font = wx.Font(12,wx.SWISS,wx.NORMAL,wx.NORMAL,False)

        conf = self.conf

        sizer = wx.GridBagSizer(4, 4)

        labstyle= wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL
        mtitle = wx.StaticText(panel, -1, 'Linear/Log Scale: ')

        logchoice = wx.Choice(panel, choices=self.conf.log_choices,  size=(200,-1))
        logchoice.SetStringSelection("x %s / y %s" % (self.conf.xscale, self.conf.yscale))
        logchoice.Bind(wx.EVT_CHOICE, self.onLogScale)

        sizer.Add(mtitle,     (1, 0), (1,1), labstyle, 2)
        sizer.Add(logchoice,  (1, 1), (1,3), labstyle, 2)


        # Zoom
        ztitle = wx.StaticText(panel, -1, 'Zoom Style: ')
        zoomchoice = wx.Choice(panel, choices=self.conf.zoom_choices, size=(200,-1))
        if self.conf.zoom_style in self.conf.zoom_choices:
            zoomchoice.SetStringSelection(self.conf.zoom_style)
        zoomchoice.Bind(wx.EVT_CHOICE, self.onZoomStyle)
        sizer.Add(ztitle,      (2, 0), (1,1), labstyle, 2)
        sizer.Add(zoomchoice,  (2, 1), (1,3), labstyle, 2)

        # Bounds
        axes = self.canvas.figure.get_axes()
        laxes = axes[0]
        raxes = None
        if len(axes) > 1:
            raxes = axes[1]
        try:
            user_lims = self.conf.user_limits[laxes]
        except:
            user_lims = 4*[None]
        auto_b  = wx.CheckBox(panel,-1, ' From Data ', (-1, -1), (-1, -1))
        auto_b.Bind(wx.EVT_CHECKBOX,self.onAutoBounds)
        try:
            auto_b.SetValue(self.conf.user_limits[laxes] == 4*[None])
        except:
            pass

        xb0, xb1 = laxes.get_xlim()
        yb0, yb1 = laxes.get_ylim()
        if user_lims[0] is not None: xb0 = user_lims[0]
        if user_lims[1] is not None: xb1 = user_lims[1]

        if user_lims[2] is not None: yb0 = user_lims[2]
        if user_lims[3] is not None: yb1 = user_lims[3]

        y2b0, y2b1 = [None, None]
        if raxes is not None:
            y2b0, y2b1 = raxes.get_ylim()
            user_lims = self.conf.user_limits[raxes]
            if user_lims[2] is not None: y2b0 = user_lims[2]
            if user_lims[3] is not None: y2b1 = user_lims[3]

        opts = dict(size=(100, -1), labeltext='', action=self.onBounds)

        self.xbounds  = [LabeledTextCtrl(panel,value=ffmt(xb0), **opts),
                         LabeledTextCtrl(panel,value=ffmt(xb1), **opts)]
        self.ybounds  = [LabeledTextCtrl(panel,value=ffmt(yb0), **opts),
                         LabeledTextCtrl(panel,value=ffmt(yb1), **opts)]
        self.y2bounds = [LabeledTextCtrl(panel,value=ffmt(y2b0), **opts),
                         LabeledTextCtrl(panel,value=ffmt(y2b1), **opts)]

        self.vpad_val = FloatSpin(panel, value=2.5, min_val=0, max_val=100,
                                  increment=0.5, digits=2,
                                  size=(FSPINSIZE, -1),
                                  action=self.onViewPadEvent)

        if user_lims == 4*[None]:
            [w.Disable() for w in self.xbounds]
            [w.Disable() for w in self.ybounds]

        if raxes is None:
            [w.Disable() for w in self.y2bounds]

        btext  = 'Plot Boundaries : '
        ptext  = ' Padding (% of Data Range): '
        xtext  = '   X axis:'
        ytext  = '   Y axis:'
        y2text = '   Y2 axis:'
        def showtext(t):
            return wx.StaticText(panel, -1, t)

        sizer.Add(showtext(btext),  (3, 0), (1, 1), labstyle, 2)
        sizer.Add(auto_b,           (3, 1), (1, 1), labstyle, 2)
        sizer.Add(showtext(ptext),  (3, 2), (1, 2), labstyle, 2)
        sizer.Add(self.vpad_val,     (3, 4), (1, 1), labstyle, 2)

        sizer.Add(showtext(xtext),  (4, 0), (1, 1), labstyle, 2)
        sizer.Add(self.xbounds[0],  (4, 1), (1, 1), labstyle, 2)
        sizer.Add(showtext(' : '),  (4, 2), (1, 1), labstyle, 2)
        sizer.Add(self.xbounds[1],  (4, 3), (1, 1), labstyle, 2)

        sizer.Add(showtext(ytext),  (5, 0), (1, 1), labstyle, 2)
        sizer.Add(self.ybounds[0],  (5, 1), (1, 1), labstyle, 2)
        sizer.Add(showtext(' : '),  (5, 2), (1, 1), labstyle, 2)
        sizer.Add(self.ybounds[1],  (5, 3), (1, 1), labstyle, 2)

        sizer.Add(showtext(y2text), (6, 0), (1, 1), labstyle, 2)
        sizer.Add(self.y2bounds[0], (6, 1), (1, 1), labstyle, 2)
        sizer.Add(showtext(' : '),  (6, 2), (1, 1), labstyle, 2)
        sizer.Add(self.y2bounds[1], (6, 3), (1, 1), labstyle, 2)

        # Margins
        _left, _top, _right, _bot = ["%.3f"% x for x in self.conf.margins]

        mtitle = wx.StaticText(panel, -1, 'Plot Margins: ')
        ltitle = wx.StaticText(panel, -1, ' Left:   ')
        rtitle = wx.StaticText(panel, -1, ' Right:  ')
        btitle = wx.StaticText(panel, -1, ' Bottom: ')
        ttitle = wx.StaticText(panel, -1, ' Top:    ')

        opts = dict(min_val=0.0, max_val=None, increment=0.01, digits=3,
                    size=(FSPINSIZE, -1), action=self.onMargins)
        lmarg = FloatSpin(panel, value=_left, **opts)
        rmarg = FloatSpin(panel, value=_right, **opts)
        bmarg = FloatSpin(panel, value=_bot, **opts)
        tmarg = FloatSpin(panel, value=_top, **opts)

        self.margins = [lmarg, tmarg, rmarg, bmarg]
        if self.conf.auto_margins:
            [m.Disable() for m in self.margins]

        auto_m  = wx.CheckBox(panel,-1, ' Default ', (-1, -1), (-1, -1))
        auto_m.Bind(wx.EVT_CHECKBOX,self.onAutoMargin)
        auto_m.SetValue(self.conf.auto_margins)

        msizer = wx.BoxSizer(wx.HORIZONTAL)
        msizer.AddMany((ltitle, lmarg, rtitle, rmarg,
                        btitle, bmarg, ttitle, tmarg))

        sizer.Add(mtitle,  (8, 0), (1,1), labstyle, 2)
        sizer.Add(auto_m,  (8, 1), (1,1), labstyle, 2)
        sizer.Add(msizer,  (9, 1), (1,6), labstyle, 2)

        autopack(panel, sizer)
        return panel

    def make_scatter_panel(self, parent, font=None):
        # list of traces
        panel = wx.Panel(parent)
        if font is None:
            font = wx.Font(12,wx.SWISS,wx.NORMAL,wx.NORMAL,False)
        sizer = wx.GridBagSizer(4, 4)

        labstyle= wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL
        slab = wx.StaticText(panel, -1, 'Symbol Size:', size=(-1,-1),style=labstyle)

        ssize = FloatSpin(panel, value=self.conf.scatter_size,
                          size=(ISPINSIZE, -1), min_val=1, max_val=500,
                          action=partial(self.onScatter, item='size'))

        sizer.Add(slab,  (0, 0), (1,1), labstyle, 5)
        sizer.Add(ssize, (0, 1), (1,1), labstyle, 5)

        conf = self.conf
        nfcol = csel.ColourSelect(panel,  -1, "",
                                  mpl_color(conf.scatter_normalcolor,
                                            default=(0, 0, 128)),
                                  size=(25, 25))
        necol = csel.ColourSelect(panel,  -1, "",
                                  mpl_color(conf.scatter_normaledge,
                                            default=(0, 0, 200)),
                                  size=(25, 25))
        sfcol = csel.ColourSelect(panel,  -1, "",
                                  mpl_color(conf.scatter_selectcolor,
                                            default=(128, 0, 0)),
                                  size=(25, 25))
        secol = csel.ColourSelect(panel,  -1, "",
                                  mpl_color(conf.scatter_selectedge,
                                           default=(200, 0, 0)),
                                  size=(25, 25))
        nfcol.Bind(csel.EVT_COLOURSELECT, partial(self.onScatter, item='scatt_nf'))
        necol.Bind(csel.EVT_COLOURSELECT, partial(self.onScatter, item='scatt_ne'))
        sfcol.Bind(csel.EVT_COLOURSELECT, partial(self.onScatter, item='scatt_sf'))
        secol.Bind(csel.EVT_COLOURSELECT, partial(self.onScatter, item='scatt_se'))

        btnstyle= wx.ALIGN_CENTER|wx.ALIGN_CENTER_VERTICAL|wx.ALL

        sizer.Add(wx.StaticText(panel, -1, 'Colors: ',
                                style=wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL),
                  (1, 0), (1,1), labstyle,2)
        sizer.Add(wx.StaticText(panel, -1, 'Normal Symbol:',
                                style=wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL),
                  (1, 1), (1,1), labstyle,2)
        sizer.Add(wx.StaticText(panel, -1, 'Selected Symbol:',
                                style=wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL),
                  (1, 2), (1,1), labstyle,2)
        sizer.Add(wx.StaticText(panel, -1, 'Face Color:',
                                style=wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL),
                  (2, 0), (1,1), labstyle,2)
        sizer.Add(wx.StaticText(panel, -1, 'Edge Color:',
                                style=wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL),
                  (3, 0), (1,1), labstyle,2)

        sizer.Add(nfcol,   (2, 1), (1,1), btnstyle,2)
        sizer.Add(necol,   (3, 1), (1,1), btnstyle,2)
        sizer.Add(sfcol,   (2, 2), (1,1), btnstyle,2)
        sizer.Add(secol,   (3, 2), (1,1), btnstyle,2)

        autopack(panel, sizer)
        return panel


    def make_text_panel(self, parent, font=None):
        panel = scrolled.ScrolledPanel(parent, size=(875, 225),
                                       style=wx.GROW|wx.TAB_TRAVERSAL, name='p1')
        if font is None:
            font = wx.Font(12,wx.SWISS,wx.NORMAL,wx.NORMAL,False)

        sizer = wx.GridBagSizer(2, 2)
        i = 0
        irow = 0
        bstyle=wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL|wx.ST_NO_AUTORESIZE
        labstyle= wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL

        ax = self.axes[0]


        self.titl = LabeledTextCtrl(panel, self.conf.title.replace('\n', '\\n'),
                                    action = partial(self.onText, item='title'),
                                    labeltext='Title: ', size=(400, -1))
        self.ylab = LabeledTextCtrl(panel, self.conf.ylabel.replace('\n', '\\n'),
                                    action = partial(self.onText, item='ylabel'),
                                    labeltext='Y Label: ', size=(400, -1))
        self.y2lab= LabeledTextCtrl(panel, self.conf.y2label.replace('\n', '\\n'),
                                    action = partial(self.onText, item='y2label'),
                                    labeltext='Y2 Label: ', size=(400, -1))
        self.xlab = LabeledTextCtrl(panel, self.conf.xlabel.replace('\n', '\\n'),
                                    action = partial(self.onText, item='xlabel'),
                                    labeltext='X Label: ', size=(400, -1))

        sizer.Add(self.titl.label,  (0, 0), (1, 1), labstyle)
        sizer.Add(self.titl,        (0, 1), (1, 4), labstyle)
        sizer.Add(self.ylab.label,  (1, 0), (1, 1), labstyle)
        sizer.Add(self.ylab,        (1, 1), (1, 4), labstyle)
        sizer.Add(self.y2lab.label, (2, 0), (1, 1), labstyle)
        sizer.Add(self.y2lab,       (2, 1), (1, 4), labstyle)
        sizer.Add(self.xlab.label,  (3, 0), (1, 1), labstyle)
        sizer.Add(self.xlab,        (3, 1), (1, 4), labstyle)

        t0 = wx.StaticText(panel, -1, 'Text Sizes:', style=labstyle)
        t1 = wx.StaticText(panel, -1, 'Titles:', style=labstyle)
        t2 = wx.StaticText(panel, -1, 'Axis Labels:',  style=labstyle)
        t3 = wx.StaticText(panel, -1, 'Legends:',  style=labstyle)

        fsopts = dict(size=(ISPINSIZE, -1), min_val=2, max_val=32, increment=0.5)
        ttl_size = FloatSpin(panel, value=self.conf.labelfont.get_size(),
                             action=partial(self.onText, item='titlesize'),
                             **fsopts)

        leg_size = FloatSpin(panel, value=self.conf.legendfont.get_size(),
                             action=partial(self.onText, item='legendsize'),
                             **fsopts)
        lab_size = FloatSpin(panel, value=self.conf.labelfont.get_size(),
                             action=partial(self.onText, item='labelsize'),
                             **fsopts)

        self.title_fontsize = ttl_size
        self.legend_fontsize = leg_size
        self.label_fontsize = lab_size

        sizer.Add(t0,        (4, 0), (1, 1), labstyle)
        sizer.Add(t1,        (4, 1), (1, 1), labstyle)
        sizer.Add(ttl_size,  (4, 2), (1, 2), labstyle)
        sizer.Add(t2,        (5, 1), (1, 1), labstyle)
        sizer.Add(lab_size,  (5, 2), (1, 2), labstyle)
        sizer.Add(t3,        (6, 1), (1, 1), labstyle)
        sizer.Add(leg_size,  (6, 2), (1, 2), labstyle)

        # Legend
        bstyle=wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL|wx.ST_NO_AUTORESIZE
        labstyle= wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL

        ax = self.axes[0]

        leg_ttl = wx.StaticText(panel, -1, 'Legend:', size=(-1, -1), style=labstyle)
        loc_ttl = wx.StaticText(panel, -1, 'Location:', size=(-1, -1), style=labstyle)
        leg_loc = wx.Choice(panel, -1, choices=self.conf.legend_locs, size=(150, -1))
        leg_loc.Bind(wx.EVT_CHOICE,partial(self.onShowLegend, item='loc'))
        leg_loc.SetStringSelection(self.conf.legend_loc)

        leg_onax = wx.Choice(panel, -1, choices=self.conf.legend_onaxis_choices,
                             size=(120, -1))
        leg_onax.Bind(wx.EVT_CHOICE,partial(self.onShowLegend, item='onaxis'))
        leg_onax.SetStringSelection(self.conf.legend_onaxis)

        togg_leg  = wx.CheckBox(panel,-1, 'Click Legend to Show/Hide Line',
                                (-1, -1), (-1, -1))
        togg_leg.Bind(wx.EVT_CHECKBOX, self.onHideWithLegend)
        togg_leg.SetValue(self.conf.hidewith_legend)

        show_leg = wx.CheckBox(panel,-1, 'Show Legend', (-1, -1), (-1, -1))
        show_leg.Bind(wx.EVT_CHECKBOX,partial(self.onShowLegend, item='legend'))
        show_leg.SetValue(self.conf.show_legend)
        if show_leg not in self.show_legend_cbs:
            self.show_legend_cbs.append(show_leg)

        show_lfr = wx.CheckBox(panel,-1, 'Show Legend Frame', (-1, -1), (-1, -1))
        show_lfr.Bind(wx.EVT_CHECKBOX,partial(self.onShowLegend,item='frame'))
        show_lfr.SetValue(self.conf.show_legend_frame)

        lsizer = wx.BoxSizer(wx.HORIZONTAL)

        lsizer.AddMany((leg_ttl, show_leg, show_lfr, togg_leg))
        sizer.Add(lsizer,    (8, 0), (1, 8), labstyle, 2)

        lsizer = wx.BoxSizer(wx.HORIZONTAL)
        lsizer.AddMany((loc_ttl, leg_loc, leg_onax))
        sizer.Add(lsizer,  (9, 1), (1, 4), labstyle, 2)
        autopack(panel, sizer)
        return panel

    def make_linetrace_panel(self, parent, font=None):
        """colours and line properties"""

        panel = scrolled.ScrolledPanel(parent, size=(900, 250),
                                       style=wx.GROW|wx.TAB_TRAVERSAL, name='p1')
        if font is None:
            font = wx.Font(12, wx.SWISS, wx.NORMAL, wx.NORMAL, False)

        sizer = wx.GridBagSizer(2, 2)
        i = 0
        cnf = self.conf
        ax = self.axes[0]

        labstyle= wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL
        opts = dict(size=(40, 30), style=labstyle)

        theme_names = list(cnf.themes.keys())
        themechoice = Choice(panel, choices=theme_names, action=self.onTheme)
        themechoice.SetStringSelection(cnf.current_theme)


        textcol = csel.ColourSelect(panel, label=" Text ", size=(80, -1),
                                    colour=mpl_color(cnf.textcolor))
        gridcol = csel.ColourSelect(panel, label=" Grid ", size=(80, -1),
                                    colour=mpl_color(cnf.gridcolor))
        bgcol = csel.ColourSelect(panel, label=" Background ", size=(120, -1),
                                  colour=mpl_color(ax.get_facecolor()))
        fbgcol = csel.ColourSelect(panel,  label=" Frame ", size=(80, -1),
                                   colour=mpl_color(self.canvas.figure.get_facecolor()))

        self.colwids = {'text': textcol, 'face': bgcol,
                        'grid': gridcol, 'frame': fbgcol}

        bgcol.Bind(csel.EVT_COLOURSELECT,   partial(self.onColor, item='face'))
        fbgcol.Bind(csel.EVT_COLOURSELECT,  partial(self.onColor, item='frame'))
        gridcol.Bind(csel.EVT_COLOURSELECT, partial(self.onColor, item='grid'))
        textcol.Bind(csel.EVT_COLOURSELECT, partial(self.onColor, item='text'))

        show_grid  = wx.CheckBox(panel,-1, ' Show Grid  ')
        show_grid.Bind(wx.EVT_CHECKBOX,self.onShowGrid)
        show_grid.SetValue(cnf.show_grid)

        show_box  = wx.CheckBox(panel,-1, ' Show Top/Right Axes  ')
        show_box.Bind(wx.EVT_CHECKBOX, self.onShowBox)
        show_box.SetValue(cnf.axes_style == 'box')

        show_leg = wx.CheckBox(panel,-1, 'Show Legend  ')
        show_leg.Bind(wx.EVT_CHECKBOX,partial(self.onShowLegend, item='legend'))
        show_leg.SetValue(cnf.show_legend)
        if show_leg not in self.show_legend_cbs:
            self.show_legend_cbs.append(show_leg)

        tsizer = wx.BoxSizer(wx.HORIZONTAL)
        tsizer.Add(wx.StaticText(panel, -1, ' Theme: '), 0, labstyle, 3)
        tsizer.Add(themechoice,  1, labstyle, 3)
        tsizer.Add(wx.StaticText(panel, -1, ' Colors: '), 0, labstyle, 3)
        tsizer.Add(textcol,   0, labstyle, 3)
        tsizer.Add(gridcol,   0, labstyle, 3)
        tsizer.Add(bgcol,     0, labstyle, 3)
        tsizer.Add(fbgcol ,   0, labstyle, 3)
        sizer.Add(tsizer,    (1, 0), (1, 9), labstyle, 3)

        tsizer = wx.BoxSizer(wx.HORIZONTAL)
        tsizer.Add(wx.StaticText(panel, -1, ' Options: '), 0, labstyle, 3)
        tsizer.Add(show_grid, 0, labstyle, 3)
        tsizer.Add(show_leg,  0, labstyle, 3)
        tsizer.Add(show_box,  0, labstyle, 3)

        sizer.Add(tsizer,    (2, 0), (1, 9), labstyle, 3)

        tsizer = wx.BoxSizer(wx.HORIZONTAL)
        tsizer.Add(wx.StaticText(panel, -1, ' All Traces:  Thickness: '), 0, labstyle, 3)
        ##
        allthk = FloatSpin(panel, size=(FSPINSIZE, -1), value=cnf.traces[0].linewidth,
                           min_val=0, max_val=20, increment=0.5, digits=1,
                           action=partial(self.onThickness, trace=-1))

        self.last_thickness_event = 0.0
        tsizer.Add(allthk, 0, labstyle, 3)
        msz = FloatSpin(panel, size=(FSPINSIZE, -1), value=cnf.traces[0].markersize,
                        min_val=0, max_val=30, increment=0.5, digits=1,
                        action=partial(self.onMarkerSize, trace=-1))
        tsizer.Add(wx.StaticText(panel, -1, ' Symbol Size: '), 0, labstyle, 3)
        tsizer.Add(msz, 0, labstyle, 3)


        sizer.Add(tsizer,    (3, 0), (1, 9), labstyle, 3)

        sizer.Add(wx.StaticText(panel, -1, 'Thing 1'), (4, 0), (1, 9), labstyle, 3)

        irow = 5

        for t in ('#','Label','Color', 'Alpha', 'Fill', 'Line Style',
                  'Thickness', 'Symbol',
                  'Size', 'Z Order', 'Join Style'):
            x = wx.StaticText(panel, -1, t)
            x.SetFont(font)
            sizer.Add(x,(irow,i),(1,1),wx.ALIGN_LEFT|wx.ALL, 3)
            i = i+1
        self.trace_labels = []
        self.choice_linewidths = []
        self.choice_markersizes = []
        ntrace_display = min(cnf.ntrace+2, len(cnf.traces))
        for i in range(ntrace_display):
            irow += 1
            label  = "trace %i" % i
            lin  = cnf.traces[i]
            dlab = lin.label
            dcol = hexcolor(lin.color)
            dthk = lin.linewidth
            dalp = lin.alpha
            dmsz = lin.markersize
            dsty = lin.style
            djsty = lin.drawstyle
            dfill = lin.fill
            dzord = lin.zorder
            dsym = lin.marker
            lab = LabeledTextCtrl(panel, dlab, size=(125, -1), labeltext="%i" % (i+1),
                                  action = partial(self.onText, item='trace', trace=i))
            self.trace_labels.append(lab)

            col = csel.ColourSelect(panel,  -1, "", dcol, size=(25, 25))
            col.Bind(csel.EVT_COLOURSELECT, partial(self.onColor, trace=i))

            self.colwids[i] = col

            thk = FloatSpin(panel, size=(FSPINSIZE, -1), value=dthk,
                            min_val=0, max_val=20, increment=0.5, digits=1,
                            action=partial(self.onThickness, trace=i))
            self.choice_linewidths.append(thk)

            sty = Choice(panel, choices=cnf.styles, size=(100,-1),
                         action=partial(self.onStyle,trace=i))
            sty.SetStringSelection(dsty)

            msz = FloatSpin(panel, size=(FSPINSIZE, -1), value=dmsz,
                            min_val=0, max_val=30, increment=0.5, digits=1,
                            action=partial(self.onMarkerSize, trace=i))
            self.choice_markersizes.append(msz)
            zor = FloatSpin(panel, size=(FSPINSIZE, -1), value=dzord,
                            min_val=-500, max_val=500, increment=1, digits=0,
                            action=partial(self.onZorder, trace=i))

            sym = Choice(panel, choices=cnf.symbols, size=(120,-1),
                         action=partial(self.onSymbol,trace=i))

            sym.SetStringSelection(dsym)

            jsty = wx.Choice(panel, -1, choices=cnf.drawstyles, size=(100,-1))
            jsty.Bind(wx.EVT_CHOICE, partial(self.onJoinStyle, trace=i))
            jsty.SetStringSelection(djsty)

            ffil = wx.CheckBox(panel, -1, '')
            ffil.Bind(wx.EVT_CHECKBOX, partial(self.onFill, trace=i))
            ffil.SetValue(dfill)

            alp = FloatSpin(panel, size=(FSPINSIZE, -1), value=dalp,
                            min_val=0, max_val=1, increment=0.05, digits=2,
                            action=partial(self.onAlpha, trace=i))


            sizer.Add(lab.label,(irow,0),(1,1),wx.ALIGN_LEFT|wx.ALL, 4)
            sizer.Add(lab, (irow,1),(1,1),wx.ALIGN_LEFT|wx.ALL, 4)
            sizer.Add(col, (irow,2),(1,1),wx.ALIGN_LEFT|wx.ALL, 4)
            sizer.Add(alp, (irow,3),(1,1),wx.ALIGN_LEFT|wx.ALL, 4)
            sizer.Add(ffil,(irow,4),(1,1),wx.ALIGN_LEFT|wx.ALL, 4)
            sizer.Add(sty, (irow,5),(1,1),wx.ALIGN_LEFT|wx.ALL, 4)
            sizer.Add(thk, (irow,6),(1,1),wx.ALIGN_LEFT|wx.ALL, 4)
            sizer.Add(sym, (irow,7),(1,1),wx.ALIGN_LEFT|wx.ALL, 4)
            sizer.Add(msz, (irow,8),(1,1),wx.ALIGN_LEFT|wx.ALL, 4)
            sizer.Add(zor, (irow,9),(1,1),wx.ALIGN_LEFT|wx.ALL, 4)
            sizer.Add(jsty, (irow,10),(1,1),wx.ALIGN_LEFT|wx.ALL, 4)


        autopack(panel,sizer)
        panel.SetupScrolling()
        return panel

    def onResetLines(self, event=None):
        pass

    def onColor(self, event=None, color=None, item='trace', trace=1, draw=True):
        event_col = event.GetValue()
        if color is None and event is not None:
            color = hexcolor(event_col)

        if item == 'trace':
            self.conf.set_trace_color(color, trace=trace)
            self.colwids[trace].SetColour(color)

        elif item == 'grid':
            self.conf.set_gridcolor(color)
        elif item == 'face':
            self.conf.set_facecolor(color)
        elif item == 'frame':
            self.conf.set_framecolor(color)
        elif item == 'text':
            self.conf.set_textcolor(color)
        if draw:
            self.canvas.draw()


    def onTheme(self, event):
        theme = event.GetString()
        conf = self.conf
        conf.set_theme(theme=theme)
        self.colwids['text'].SetColour(conf.textcolor)
        self.colwids['grid'].SetColour(conf.gridcolor)
        self.colwids['face'].SetColour(conf.facecolor)
        self.colwids['frame'].SetColour(conf.framecolor)

        self.title_fontsize.SetValue(self.conf.titlefont.get_size())
        self.legend_fontsize.SetValue(self.conf.legendfont.get_size())

        ntrace_display = min(self.conf.ntrace+2, len(self.conf.traces))
        for i in range(ntrace_display):
            try:
                lin = self.conf.traces[i]
                curcol = hexcolor(self.colwids[i].GetColour())
                newcol = hexcolor(lin.color)
                self.colwids[i].SetColour(newcol)
                if newcol != curcol:
                    self.conf.set_trace_color(newcol, trace=i)
            except KeyError:
                pass
        conf.draw_legend()

    def onLogScale(self, event):
        xword, yword = event.GetString().split(' / ')
        xscale = xword.replace('x', '').strip()
        yscale = yword.replace('y', '').strip()
        self.conf.set_logscale(xscale=xscale, yscale=yscale)

    def onZoomStyle(self, event=None):
        self.conf.zoom_style = event.GetString()

    def onStyle(self, event, trace=0):
        self.conf.set_trace_style(event.GetString(),trace=trace)

    def onJoinStyle(self, event, trace=0):
        self.conf.set_trace_drawstyle(event.GetString(), trace=trace)

    def onFill(self, event, trace=0):
        self.conf.set_trace_fill(event.IsChecked(), trace=trace)

    def onSymbol(self, event, trace=0):
        self.conf.set_trace_marker(event.GetString(), trace=trace)

    def onMarkerSize(self, event, trace=0):
        val = event.GetEventObject().GetValue()
        if trace == -1:
            for t, c in enumerate(self.choice_markersizes):
                c.SetValue(val)
                self.conf.set_trace_markersize(val, trace=t)
        else:
            self.conf.set_trace_markersize(val, trace=trace)

    def onAlpha(self, event, trace=0):
        self.conf.set_trace_alpha(event.GetEventObject().GetValue(), trace=trace,
                                  delay_draw=False)

    def onZorder(self, event, trace=0):
        self.conf.set_trace_zorder(event.GetEventObject().GetValue(),
                                   trace=trace)

    def onThickness(self, event, trace=0):
        val = event.GetEventObject().GetValue()
        if trace == -1:
            for t, c in enumerate(self.choice_linewidths):
                c.SetValue(val)
                self.conf.set_trace_linewidth(val, trace=t)
        else:
            self.conf.set_trace_linewidth(val, trace=trace)


    def onAutoBounds(self,event):
        axes = self.canvas.figure.get_axes()
        if event.IsChecked():
            for ax in axes:
                self.conf.user_limits[ax] = [None, None, None, None]
            [m.Disable() for m in self.xbounds]
            [m.Disable() for m in self.ybounds]
            [m.Disable() for m in self.y2bounds]
            self.vpad_val.Enable()
            self.conf.unzoom(full=True)
        else:
            self.vpad_val.Disable()
            xb = axes[0].get_xlim()
            yb = axes[0].get_ylim()
            for m, v in zip(self.xbounds, xb):
                m.Enable()
                m.SetValue(ffmt(v))
            for m, v in zip(self.ybounds, yb):
                m.Enable()
                m.SetValue(ffmt(v))
            if len(axes) > 1:
                y2b = axes[1].get_ylim()
                for m, v in zip(self.y2bounds, y2b):
                    m.Enable()
                    m.SetValue(ffmt(v))

    def onBounds(self, event=None):
        def FloatNone(v):
            if v in ('', 'None', 'none'):
                return None
            try:
                return float(v)
            except:
                return None

        axes = self.canvas.figure.get_axes()
        xmin, xmax = [FloatNone(w.GetValue()) for w in self.xbounds]
        ymin, ymax = [FloatNone(w.GetValue()) for w in self.ybounds]
        self.conf.user_limits[axes[0]] = [xmin, xmax, ymin, ymax]

        if len(axes) > 1:
            y2min, y2max = [FloatNone(w.GetValue()) for w in self.y2bounds]
            self.conf.user_limits[axes[1]] = [xmin, xmax, y2min, y2max]
        self.conf.set_viewlimits()
        self.conf.canvas.draw()


    def onAutoMargin(self,event):
        self.conf.auto_margins = event.IsChecked()
        if self.conf.auto_margins:
            [m.Disable() for m in self.margins]
        else:
            ppanel = self.GetParent()
            vals = ppanel.get_default_margins()
            for m, v in zip(self.margins, vals):
                m.Enable()
                m.SetValue(v)
        self.conf.canvas.draw()

    def onMargins(self, event=None):
        left, top, right, bottom = [float(w.GetValue()) for w in self.margins]
        self.conf.set_margins(left=left, top=top, right=right, bottom=bottom)

    def onViewPadEvent(self, event=None):

        self.conf.viewpad = float(self.vpad_val.GetValue())
        self.conf.set_viewlimits()
        self.conf.canvas.draw()

        axes = self.canvas.figure.get_axes()
        xb = axes[0].get_xlim()
        yb = axes[0].get_ylim()
        for m, v in zip(self.xbounds, xb):
            m.SetValue(ffmt(v))

        for m, v in zip(self.ybounds, yb):
            m.SetValue(ffmt(v))

    def onScatter(self, event, item=None):
        conf = self.conf
        axes = self.canvas.figure.get_axes()[0]
        if item == 'size':
            conf.scatter_size = event.GetInt()
        elif item == 'scatt_nf':
            self.conf.scatter_normalcolor = hexcolor(event.GetValue())
        elif item == 'scatt_ne':
            self.conf.scatter_normaledge = hexcolor(event.GetValue())
        elif item == 'scatt_sf':
            self.conf.scatter_selectcolor = hexcolor(event.GetValue())
        elif item == 'scatt_se':
            self.conf.scatter_selectedge = hexcolor(event.GetValue())

        axes.cla()
        xd, yd = conf.scatter_xdata, conf.scatter_ydata
        sdat = zip(xd, yd)
        mask = conf.scatter_mask
        if mask is  None:
            axes.scatter(xd, yd, s=conf.scatter_size,
                         c=conf.scatter_normalcolor,
                         edgecolors=conf.scatter_normaledge)
        else:
            axes.scatter(xd[np.where(~mask)], yd[np.where(~mask)],
                         s=conf.scatter_size,
                         c=conf.scatter_normalcolor,
                         edgecolors=conf.scatter_normaledge)
            axes.scatter(xd[np.where(mask)], yd[np.where(mask)],
                         s=conf.scatter_size,
                         c=conf.scatter_selectcolor,
                         edgecolors=conf.scatter_selectedge)
        self.conf.relabel(delay_draw=False)

    def onText(self, event=None, item='trace', trace=0):
        if item=='labelsize':
            size = self.label_fontsize.GetValue()
            self.conf.labelfont.set_size(size)
            for ax in self.axes:
                for lab in ax.get_xticklabels()+ax.get_yticklabels():
                    lab.set_fontsize(size)
            self.conf.relabel()
            return
        elif item=='titlesize':
            size = self.title_fontsize.GetValue()
            self.conf.titlefont.set_size(size)
            self.conf.relabel()
            return
        elif item=='legendsize':
            size = self.legend_fontsize.GetValue()
            self.conf.legendfont.set_size(size)
            self.conf.draw_legend()
            return

        if item == 'title':
            wid = self.titl
        elif item == 'ylabel':
            wid = self.ylab
        elif item == 'y2label':
            wid = self.y2lab
        elif item == 'xlabel':
            wid = self.xlab
        elif item == 'trace':
            wid = self.trace_labels[trace]

        s = wid.GetValue()
        try:
            s = str(s).strip()
        except TypeError:
            s = ''

        if '\\' in s and '$' in s:
            s = clean_texmath(s)
            # print(" s = ", s)
        if item in ('xlabel', 'ylabel', 'y2label', 'title'):
            try:
                kws = {item: s}
                self.conf.relabel(**kws)
                wid.SetBackgroundColour((255, 255, 255))
            except: # as from latex error!
                wid.SetBackgroundColour((250, 250, 200))
        elif item == 'trace':
            try:
                self.conf.set_trace_label(s, trace=trace)
            except:
                pass

    def onShowGrid(self,event):
        self.conf.enable_grid(show=event.IsChecked())

    def onShowBox(self, event=None):
        style='box'
        if not event.IsChecked(): style='open'
        self.conf.set_axes_style(style=style)

    def onShowLegend(self, event, item=''):
        auto_location = True
        if item == 'legend':
            self.conf.show_legend  = checked = event.IsChecked()
            for cb in self.show_legend_cbs:
                if cb.GetValue() != checked:
                    cb.SetValue(checked)

        elif item=='frame':
            self.conf.show_legend_frame = event.IsChecked()
        elif item=='loc':
            self.conf.legend_loc  = event.GetString()
            auto_location = False
        elif item=='onaxis':
            self.conf.legend_onaxis  = event.GetString()
        self.conf.draw_legend(auto_location=auto_location)

    def onDragLegend(self, event=None):
        self.conf.draggable_legend = event.IsChecked()
        self.conf.draw_legend()

    def onHideWithLegend(self, event=None):
        self.conf.hidewith_legend = event.IsChecked()
        self.conf.draw_legend()

    def redraw_legend(self):
        self.conf.draw_legend()

    def onExit(self, event):
        self.Close(True)



#!/usr/bin/python
import numpy as np
import wxmplot.interactive as wi

x = np.linspace(0.0, 20.0, 201)
y1 = np.sin(x)/(x+1)
dy1 = 0.04 * np.ones(201)

y2 = np.cos(x*1.1)/(x+1)
dy2 = 0.07 * np.ones(201)

wi.plot(x, y1, dy=dy1, label='thing1', alpha=0.25, fill=True, xlabel='x (mm)')
wi.plot(x, y2, dy=dy2, label='thing2', alpha=0.40, fill=True, show_legend=True)



#!/usr/bin/python
#
# compare to plplot example at
#  http://plplot.sourceforge.net/examples.php?demo=00&lbind=Python
##
## def main(w):
##    w.plenv( 0, 100, 0, 1, 0, 0)
##    w.pllab( "x", "y=100 x#u2#d", "Simple PLplot demo of a 2D line plot" )
##    w.plline( x, y )


import numpy as np

x = np.linspace(0, 1, 101)
y = 100*x**2

import wxmplot.interactive as wi
wi.plot(x, y, color='red', xlabel='x', ylabel=r'$y=100 x^2$',
        title="Simple PLplot demo of a 2D line plot", theme='dark')        





#!/usr/bin/python

from wxmplot import PlotApp
from datetime import datetime
f = open('time.dat','r')
l = f.readlines()
f.close()
t = []
x = []
for i in l:
    j =  i[:-1].strip().split()
    t.append(datetime.fromtimestamp(float(j[0])))
    x.append(float(j[1]))


app = PlotApp()
app.plot(t, x, use_dates=True, drawstyle='steps-post', marker='+')
app.set_title('Time series data:')
app.run()



#!/usr/bin/python
#
# scatterplot example, with lassoing and
# a user-level lasso-callback
import sys
import wx
import wxmplot
import numpy

x   = numpy.arange(100)/20.0 + numpy.random.random(size=100)
y   = numpy.random.random(size=len(x))
def onlasso(data=None, selected=None, mask=None):
    print( ':: lasso ', selected)

app = wx.App()

pframe = wxmplot.PlotFrame()
pframe.scatterplot(x, y, title='Scatter Plot', size=15,
                   xlabel='$ x\, \mathrm{(\AA)}$',
                   ylabel='$ y\, \mathrm{(\AA^{-1})}$')
pframe.panel.lasso_callback = onlasso
pframe.write_message('WXMPlot PlotFrame example: Try Help->Quick Reference')
pframe.Show()
#
app.MainLoop()



import wx
import wx.lib.scrolledpanel as scrolled
import wx.lib.colourselect as csel

from math import log10

import numpy as np

import matplotlib.cm as cmap
from matplotlib.ticker import FuncFormatter, ScalarFormatter

from wxutils import get_cwd
from .colors import register_custom_colormaps, hexcolor, hex2rgb, mpl_color
from .config import bool_ifnotNone, ifnotNone
from .plotconfigframe import FNB_STYLE, autopack
from .utils import  LabeledTextCtrl, SimpleText, Check, Choice, HLine, pack, FloatSpin, MenuItem

try:
    import yaml
    HAS_YAML = True
except ImportError:
    HAS_YAML = False

cm_names = register_custom_colormaps()

ColorMap_List = []

for cm in ('gray', 'coolwarm', 'viridis', 'inferno', 'plasma', 'magma', 'red',
           'green', 'blue', 'magenta', 'yellow', 'cyan', 'Reds', 'Greens',
           'Blues', 'cool', 'hot', 'copper', 'red_heat', 'green_heat',
           'blue_heat', 'spring', 'summer', 'autumn', 'winter', 'ocean',
           'terrain', 'jet', 'stdgamma', 'hsv', 'Accent', 'Spectral', 'PiYG',
           'PRGn', 'Spectral', 'YlGn', 'YlGnBu', 'RdBu', 'RdPu', 'RdYlBu',
           'RdYlGn'):

    if cm in cm_names or hasattr(cmap, cm):
        ColorMap_List.append(cm)


Contrast_List = ['None', '0.01', '0.02', '0.05', '0.1', '0.2', '0.5', '1.0',
                 '2.0', '5.0']

Contrast_NDArray = np.array((-1.0, 0.01, 0.02, 0.05, 0.1, 0.2, 0.5, 1., 2, 5.))

Interp_List = ('nearest', 'bicubic', 'quadric', 'gaussian', 'kaiser',
               'bessel', 'mitchell', 'catrom', 'spline16', 'spline36',
               'bilinear', 'hanning', 'hamming', 'hermite', 'sinc', 'lanczos')

Slices_List = ('None', 'X', 'Y')

RGB_COLORS = ('red', 'green', 'blue')

class ImageConfig:
    def __init__(self, axes=None, fig=None, canvas=None):
        self.axes   = axes
        self.fig  = fig
        self.canvas  = canvas
        self.cmap  = [cmap.gray, cmap.gray, cmap.gray]
        self.cmap_reverse = False
        self.interp = 'nearest'
        self.show_axis = False
        self.show_grid = False
        self.grid_color = '#807030'
        self.grid_alpha = 0.25
        self.log_scale = False
        self.flip_ud = False
        self.flip_lr = False
        self.rot_level  = 0
        self.contrast_level = 0
        self.datalimits = [None, None, None, None]
        self.cmap_lo = [0, 0, 0]
        self.cmap_range = 1000
        self.cmap_hi = [1000, 1000, 1000]
        self.tricolor_bg = 'black'
        self.tricolor_mode = 'rgb'
        self.int_lo = [0, 0, 0]
        self.int_hi = [1, 1, 1]
        self.data = None
        self.xdata = None
        self.ydata = None
        self.xlab = 'X'
        self.ylab = 'Y'
        self.indices = None
        self.title = 'image'
        self.style = 'image'
        self.highlight_areas = []
        self.ncontour_levels = 10
        self.contour_levels = None
        self.contour_labels = True
        self.cursor_mode = 'zoom'
        self.zoombrush = wx.Brush('#040410',  wx.SOLID)
        self.zoompen   = wx.Pen('#101090',  3, wx.SOLID)
        self.zoom_choices = ('both x and y', 'x only', 'y only')
        self.zoom_style = 'both'
        self.zoom_lims = []
        self.slices = Slices_List[0]
        self.slice_xy = -1, -1
        self.slice_width = 1
        self.slice_onmotion = False
        self.scalebar_show = False
        self.scalebar_showlabel = False
        self.scalebar_label = ''
        self.scalebar_textoffset =  10, 25
        self.scalebar_pos =  5, 5
        self.scalebar_size = 1, 1
        self.scalebar_pixelsize = None, None
        self.scalebar_units = 'mm'
        self.scalebar_color = '#EEEE99'
        self._xfmt = None
        self._yfmt = None
        self.set_formatters()


    def set_colormap(self, name, reverse=False, icol=0):
        self.cmap_reverse = reverse
        if reverse and not name.endswith('_r'):
            name = name + '_r'
        elif not reverse and name.endswith('_r'):
            name = name[:-2]
        self.cmap[icol] = curr_cmap = cmap.get_cmap(name)
        if not hasattr(curr_cmap, '_lut'):
            try:
                curr_cmap._init()
            except:
                pass

        if hasattr(self, 'contour'):
            xname = 'gray'
            if name == 'gray_r':
                xname = 'Reds_r'
            elif name == 'gray':
                xname = 'Reds'
            elif name.endswith('_r'):
                xname = 'gray_r'
            self.contour.set_cmap(getattr(cmap, xname))
        if hasattr(self, 'image'):
            self.image.set_cmap(curr_cmap)

        if hasattr(self, 'highlight_areas') and hasattr(curr_cmap, '_lut'):
            rgb  = [int(i*240)^255 for i in curr_cmap._lut[0][:3]]
            col  = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
            for area in self.highlight_areas:
                for w in area.labelTexts:
                    w.set_color(col)
                for w in area.collections:
                    w.set_edgecolor(col)

    def flip_vert(self):
        "flip image along vertical axis (up/down)"
        self.data = np.flipud(self.data)
        if self.ydata is not None:
            self.ydata = self.ydata[::-1]
        self.flip_ud = not self.flip_ud

    def flip_horiz(self):
        "flip image along horizontal axis (left/right)"
        self.data = np.fliplr(self.data)
        if self.xdata is not None:
            self.xdata = self.xdata[::-1]
        self.flip_lr = not self.flip_lr

    def rotate90(self, event=None):
        "rotate 90 degrees, CW"
        if self.xdata is not None:
            self.xdata = self.xdata[::-1]
        if self.ydata is not None:
            self.ydata = self.ydata[:]
        self.xdata, self.ydata = self.ydata, self.xdata
        self.xlab, self.ylab = self.ylab, self.xlab
        self.data = np.rot90(self.data)
        self.rot_level += 1
        if self.rot_level == 4:
            self.rot_level = 0

    def reset_formats(self):
        "reset formats for x/y axis"
        self._xfmt = self._yfmt = None

    def set_formatters(self):
        if self.axes is not None:
            self.axes.xaxis.set_major_formatter(FuncFormatter(self.xformatter))
            self.axes.yaxis.set_major_formatter(FuncFormatter(self.yformatter))

    def xformatter(self, x, pos):
        " x-axis formatter "
        return self._format(x, pos, dtype='x')

    def yformatter(self, y, pos):
        " y-axis formatter "
        return self._format(y, pos, dtype='y')

    def set_format_str(self, axis, dat):
        try:
            ticks = axis.get_major_locator()()
        except:
            ticks = [0, 1]

        dticks = []
        for t in ticks:
            try:
                dticks.append(dat[int(t)])
            except:
                pass
        if len(dticks) < 2:
            dticks.append(dat.min())
            dticks.append(dat.max())

        dstep = max(2.e-15, abs(np.diff(dticks).mean()))
        if dstep > 5e4 or (dstep < 5.e-4 and dticks.mean() < 5.e-2):
            fmt = '%.2e'
        else:
            ndigs = max(0, 3 - round(log10(dstep)))
            while ndigs >= 0:
                if np.abs(dticks- np.round(dticks, decimals=ndigs)).max() < 2e-3*dstep:
                    ndigs -= 1
                else:
                    break
            fmt = '%%1.%df' % min(9, ndigs+1)
        return fmt

    def _format(self, x, pos, dtype='x'):
        """ home built tick formatter to use with FuncFormatter():
        x     value to be formatted
        type  'x' or 'y' or 'y2' to set which list of ticks to get

        also sets self._yfmt/self._xfmt for statusbar
        """
        if dtype == 'y':
            ax = self.axes.yaxis
            dat  = self.ydata
            if dat is None:
                dat = np.arange(self.data.shape[0])
            if self._yfmt is None:
                self._yfmt = self.set_format_str(ax, dat)
            fmt = self._yfmt

        else:
            ax = self.axes.xaxis
            dat = self.xdata
            if dat is None:
                dat = np.arange(self.data.shape[1])
            if self._xfmt is None:
                self._xfmt = self.set_format_str(ax, dat)
            fmt = self._xfmt

        try:
            s =  fmt % dat[int(x)]
        except:
            s = ''
        s.strip()
        s = s.replace('+', '')
        while s.find('e0')>0:
            s = s.replace('e0','e')
        while s.find('-0')>0:
            s = s.replace('-0','-')
        return s

    def relabel(self):
        " re draw labels (title, x,y labels)"
        pass

    def set_zoombrush(self,color, style):
        self.zoombrush = wx.Brush(color, style)

    def set_zoompen(self,color, style):
        self.zoompen = wx.Pen(color, 3, style)

    def tricolor_white_bg(self, img):
        """transforms image from RGB with (0,0,0)
        showing black to  RGB with 0,0,0 showing white

        takes the Red intensity and sets
        the new intensity to go
        from (0, 0.5, 0.5) (for Red=0)  to (0, 0, 0) (for Red=1)
        and so on for the Green and Blue maps.

        Thus the image will be transformed from
          old intensity                new intensity
          (0.0, 0.0, 0.0) (black)   (1.0, 1.0, 1.0) (white)
          (1.0, 1.0, 1.0) (white)   (0.0, 0.0, 0.0) (black)
          (1.0, 0.0, 0.0) (red)     (1.0, 0.5, 0.5) (red)
          (0.0, 1.0, 0.0) (green)   (0.5, 1.0, 0.5) (green)
          (0.0, 0.0, 1.0) (blue)    (0.5, 0.5, 1.0) (blue)

        """
        tmp = 0.5*(1.0 - (img - img.min())/(img.max() - img.min()))
        out = tmp*0.0
        out[:,:,0] = tmp[:,:,1] + tmp[:,:,2]
        out[:,:,1] = tmp[:,:,0] + tmp[:,:,2]
        out[:,:,2] = tmp[:,:,0] + tmp[:,:,1]
        return out

    def rgb2cmy(self, img, whitebg=False):
        """transforms image from RGB to CMY"""
        tmp = img*1.0
        if whitebg:
            tmp = (1.0 - (img - img.min())/(img.max() - img.min()))
        out = tmp*0.0
        out[:,:,0] = (tmp[:,:,1] + tmp[:,:,2])/2.0
        out[:,:,1] = (tmp[:,:,0] + tmp[:,:,2])/2.0
        out[:,:,2] = (tmp[:,:,0] + tmp[:,:,1])/2.0
        return out

    def set_config(self, interp=None, colormap=None, reverse_colormap=None,
                   contrast_level=None, flip_ud=None, flip_lr=None,
                   rot=None, tricolor_bg=None, ncontour_levels=None,
                   title=None, style=None):
        """set configuration options:

           interp, colormap, reverse_colormap, contrast_levels, flip_ud,
           flip_lr, rot, tricolor_bg, ncontour_levels, title, style
        """
        if interp is not None:
            interp = interp.lower()
            self.interp = interp if interp in Interp_List else self.interp

        if colormap is not None:
            colormap = colormap.lower()
            if colormap.endswith('_r'):
                reverse_colormap = True
                colormap = colormap[:-2]
            self.colormap = colormap if colormap in ColorMap_List else self.colormap

        if contrast_level is not None:
            self.contrast_level = float(contrast_level)

        self.cmap_reverse = bool_ifnotNone(reverse_colormap, self.cmap_reverse)
        self.flip_ud = bool_ifnotNone(flip_ud, self.flip_ud)
        self.flip_lr = bool_ifnotNone(flip_lr, self.flip_lr)
        self.rot     = bool_ifnotNone(rot, self.rot)

        if tricolor_bg is not None:
            tricolor_bg = tricolor_bg.lower()
            if tricolor_bg in ('black', 'white'):
                self.tricolor_bg = tricolor_bg

        if ncontour_levels is not None:
            self.ncontour_level = int(ncontour_levels)

        if style is not None:
            style = style.lower()
            if style in ('image', 'contour'):
                self.style = style

        self.title = ifnotNone(title, self.title)


    def get_config(self):
        """get dictionary of configuration options"""
        out = {'reverse_colormap': self.cmap_reverse}
        for attr in ('interp', 'colormap', 'contrast_levels', 'flip_ud',
                     'flip_lr', 'rot', 'tricolor_bg', 'ncontour_levels',
                     'title', 'style'):
            out[attr] = getattr(self, attr)
        return out

labstyle= wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL|wx.ALL

class ImageConfigFrame(wx.Frame):
    """ GUI Configure Frame for Images"""
    def __init__(self, parent=None, config=None, trace_color_callback=None):
        if config is None:
            config = ImageConfig()
        self.conf   = config
        self.parent = parent
        self.canvas = self.conf.canvas
        self.axes = self.canvas.figure.get_axes()
        self.DrawPanel()
        mbar = wx.MenuBar()
        fmenu = wx.Menu()
        MenuItem(self, fmenu, "Save Configuration\tCtrl+S",
                 "Save Configuration",
                 self.save_config)
        MenuItem(self, fmenu, "Load Configuration\tCtrl+R",
                 "Load Configuration",
                 self.load_config)
        mbar.Append(fmenu, 'File')
        self.SetMenuBar(mbar)

    def save_config(self, evt=None, fname='wxmplot.yaml'):
        if not HAS_YAML:
            return
        file_choices = 'YAML Config File (*.yaml)|*.yaml'
        dlg = wx.FileDialog(self, message='Save image configuration',
                            defaultDir=get_cwd(),
                            defaultFile=fname,
                            wildcard=file_choices,
                            style=wx.FD_SAVE|wx.FD_CHANGE_DIR)

        if dlg.ShowModal() == wx.ID_OK:
            conf = self.conf.get_current_config()
            ppath = os.path.abspath(dlg.GetPath())
            with open(ppath, 'w') as fh:
                fh.write("%s\n" % yaml.dump(conf))


    def load_config(self, evt=None):
        if not HAS_YAML:
            return
        file_choices = 'YAML Config File (*.yaml)|*.yaml'
        dlg = wx.FileDialog(self, message='Read image configuration',
                            defaultDir=get_cwd(),
                            wildcard=file_choices,
                            style=wx.FD_OPEN)

        if dlg.ShowModal() == wx.ID_OK:
            conf = yaml.safe_load(open(os.path.abspath(dlg.GetPath()), 'r').read())
            self.conf.load_config(conf)

    def DrawPanel(self):
        style = wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, self.parent, -1, 'Configure Image', style=style)

        conf = self.conf

        self.SetFont(wx.Font(12,wx.SWISS,wx.NORMAL,wx.NORMAL,False))
        self.SetBackgroundColour(hex2rgb('#FEFEFE'))

        sizer = wx.GridBagSizer(2, 2)
        irow = 0
        bstyle=wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL|wx.ST_NO_AUTORESIZE


        # contours
        ctitle = SimpleText(self, 'Contours:', colour='#DD0000')
        label = SimpleText(self,  "# Levels:")

        self.ncontours = FloatSpin(self, value=conf.ncontour_levels,
                                     min_val=0, max_val=5000,
                                     increment=1, digits=0, size=(60, -1),
                                     action=self.onContourEvents)

        self.showlabels = Check(self, label='Show Labels?',
                                default=conf.contour_labels,
                                action=self.onContourEvents)

        sizer.Add(ctitle,          (irow, 0), (1, 2), labstyle, 2)
        irow += 1
        sizer.Add(label,           (irow, 0), (1, 1), labstyle, 2)
        sizer.Add(self.ncontours,  (irow, 1), (1, 1), labstyle, 2)
        sizer.Add(self.showlabels, (irow, 2), (1, 1), labstyle, 2)
        irow += 1
        sizer.Add(HLine(self, size=(500, -1)), (irow, 0), (1, 4), labstyle, 2)

        # Grid
        title = SimpleText(self, 'Image Grid:', colour='#DD0000')
        label_gcolor = SimpleText(self, "Color:")
        label_galpha = SimpleText(self, "Alpha:")
        self.show_grid = Check(self, label='Show Grid with Labeled Axes?',
                               default=conf.show_grid,
                               action=self.onGridEvents)

        self.grid_alpha = FloatSpin(self, value=conf.grid_alpha,
                                     min_val=0, max_val=1,
                                     increment=0.05, digits=3, size=(130, -1),
                                     action=self.onGridEvents)
        self.grid_color = csel.ColourSelect(self,  -1, "",
                                            mpl_color(conf.grid_color),
                                            size=(50, -1))
        self.grid_color.Bind(csel.EVT_COLOURSELECT, self.onGridEvents)

        irow += 1
        sizer.Add(title,          (irow, 0), (1, 1), labstyle, 2)
        sizer.Add(self.show_grid, (irow, 1), (1, 1), labstyle, 2)
        irow += 1
        sizer.Add(label_gcolor,     (irow, 0), (1, 1), labstyle, 2)
        sizer.Add(self.grid_color,  (irow, 1), (1, 1), labstyle, 2)
        sizer.Add(label_galpha,     (irow, 2), (1, 1), labstyle, 2)
        sizer.Add(self.grid_alpha,  (irow, 3), (1, 1), labstyle, 2)

        irow += 1
        sizer.Add(HLine(self, size=(500, -1)), (irow, 0), (1, 4), labstyle, 2)


        # X/Y Slices
        title =  SimpleText(self, 'X/Y Slices:', colour='#DD0000')
        label_dir = SimpleText(self, "Direction:")
        label_wid = SimpleText(self, "Width (pixels):")
        self.slice_width = FloatSpin(self, value=conf.slice_width,
                                     min_val=0, max_val=5000,
                                     increment=1, digits=0, size=(60, -1),
                                     action=self.onSliceEvents)
        self.slice_dir =  Choice(self, size=(90, -1),
                                 choices=Slices_List,
                                 action=self.onSliceEvents)
        self.slice_dir.SetStringSelection(conf.slices)

        self.slice_dynamic = Check(self,label='Slices Follow Mouse Motion?',
                                   default=conf.slice_onmotion,
                                   action=self.onSliceEvents)
        irow += 1
        sizer.Add(title,            (irow, 0), (1, 1), labstyle, 2)
        sizer.Add(self.slice_dynamic, (irow, 1), (1, 2), labstyle, 2)
        irow += 1
        sizer.Add(label_dir,        (irow, 0), (1, 1), labstyle, 2)
        sizer.Add(self.slice_dir,   (irow, 1), (1, 1), labstyle, 2)
        sizer.Add(label_wid,        (irow, 2), (1, 1), labstyle, 2)
        sizer.Add(self.slice_width, (irow, 3), (1, 1), labstyle, 2)

        irow += 1
        sizer.Add(HLine(self, size=(500, -1)), (irow, 0), (1, 4), labstyle, 2)

        # Scalebar
        ypos, xpos = conf.scalebar_pos
        tyoff, txoff = conf.scalebar_textoffset
        ysiz, xsiz = conf.scalebar_size
        units = conf.scalebar_units
        dshape = conf.data.shape
        ystep, xstep = conf.scalebar_pixelsize
        if xstep is None or ystep is None:
            ystep, xstep = 1, 1
            if conf.xdata is not None:
                xstep = abs(np.diff(conf.xdata).mean())
            if conf.ydata is not None:
                ystep = abs(np.diff(conf.ydata).mean())
            conf.scalebar_pixelsize = ystep, xstep

        title =  SimpleText(self, 'Scalebar:', colour='#DD0000')


        lab_opts = dict(size=(120, -1))
        color_label = SimpleText(self, 'Color: ')
        xpos_label = SimpleText(self, 'X Position: ')
        ypos_label = SimpleText(self, 'Y Position: ')
        size_label = SimpleText(self, 'Scalebar Size: ')
        pos_label = SimpleText(self, 'Scalebar Position (pixels from lower left):')
        toff_label = SimpleText(self, 'Scalebar Label Position (pixels from scalebar):')
        xoff_label = SimpleText(self, 'X Offset: ')
        yoff_label = SimpleText(self, 'Y Offset: ')

        width_label = SimpleText(self, 'Width (%s): ' % units)
        height_label = SimpleText(self, 'Height (pixels): ')
        pixsize_label = SimpleText(self, 'Pixel Size: ')
        xpix_label = SimpleText(self, 'X pixelsize: ')
        ypix_label = SimpleText(self, 'Y pixelsize: ')


        self.pixunits = LabeledTextCtrl(self, value=conf.scalebar_units,
                                        size=(80, -1),
                                        labeltext='Units:',
                                        action=self.onScalebarEvents)

        self.show_scalebar = Check(self, label='Show Scalebar',
                                   default=conf.scalebar_show,
                                   action=self.onScalebarEvents)

        self.show_label = Check(self, label='Show Label?',
                                default=conf.scalebar_showlabel,
                                action=self.onScalebarEvents)


        stext = "Image Size: X=%d, Y=%d pixels" % (dshape[1], dshape[0])

        scale_text = SimpleText(self, label=stext)

        self.label  = LabeledTextCtrl(self, value=conf.scalebar_label,
                                      size=(150, -1),
                                      labeltext='Label:',
                                      action=self.onScalebarEvents)

        self.color = csel.ColourSelect(self,  -1, "",
                                       mpl_color(conf.scalebar_color),
                                       size=(50, -1))
        self.color.Bind(csel.EVT_COLOURSELECT, self.onScalebarEvents)



        opts = dict(increment=1, digits=0, size=(100, -1),
                    action=self.onScalebarEvents)

        self.xpos = FloatSpin(self,  value=xpos, max_val=dshape[1],
                              min_val=0, **opts)
        self.ypos = FloatSpin(self,  value=ypos, max_val=dshape[0],
                              min_val=0, **opts)
        self.height = FloatSpin(self, value=ysiz, max_val=dshape[0],
                                min_val=0, **opts)

        self.txoff = FloatSpin(self,  value=txoff, max_val=dshape[1],
                               min_val=-dshape[1], **opts)
        self.tyoff = FloatSpin(self,  value=tyoff, max_val=dshape[0],
                               min_val=-dshape[0], **opts)


        opts['increment'] = xstep
        opts['digits'] = max(1, 2 - int(np.log10(abs(xstep))))
        self.width = FloatSpin(self, value=xsiz, max_val=dshape[1]*xstep, **opts)

        opts['increment'] = 0.001
        opts['digits'] = 5
        self.xpix = FloatSpin(self, value=xstep, **opts)
        self.ypix = FloatSpin(self, value=ystep, **opts)


        irow += 1
        sizer.Add(title,           (irow, 0), (1, 1), labstyle, 2)
        sizer.Add(scale_text,      (irow, 1), (1, 4), labstyle, 2)


        irow += 1
        sizer.Add(pixsize_label,       (irow, 0), (1, 1), labstyle, 2)
        sizer.Add(self.pixunits.label, (irow, 1), (1, 1), labstyle, 2)
        sizer.Add(self.pixunits,       (irow, 2), (1, 1), labstyle, 2)

        irow += 1
        sizer.Add(xpix_label,      (irow, 0), (1, 1), labstyle, 2)
        sizer.Add(self.xpix,       (irow, 1), (1, 1), labstyle, 2)
        sizer.Add(ypix_label,      (irow, 2), (1, 1), labstyle, 2)
        sizer.Add(self.ypix,       (irow, 3), (1, 1), labstyle, 2)

        irow += 1
        sizer.Add(HLine(self, size=(500, -1)), (irow, 0), (1, 4), labstyle, 2)
        irow += 1

        sizer.Add(size_label,       (irow, 0), (1, 3), labstyle, 2)

        irow += 1
        sizer.Add(width_label,     (irow, 0), (1, 1), labstyle, 2)
        sizer.Add(self.width,      (irow, 1), (1, 1), labstyle, 2)
        sizer.Add(height_label,    (irow, 2), (1, 1), labstyle, 2)
        sizer.Add(self.height,     (irow, 3), (1, 1), labstyle, 2)

        irow += 1
        sizer.Add(HLine(self, size=(500, -1)), (irow, 0), (1, 4), labstyle, 2)

        irow += 1
        sizer.Add(pos_label,     (irow, 0), (1, 3), labstyle, 2)

        irow += 1
        sizer.Add(xpos_label,   (irow, 0), (1, 1), labstyle, 2)
        sizer.Add(self.xpos,    (irow, 1), (1, 1), labstyle, 2)
        sizer.Add(ypos_label,   (irow, 2), (1, 1), labstyle, 2)
        sizer.Add(self.ypos,    (irow, 3), (1, 1), labstyle, 2)


        irow += 1
        sizer.Add(HLine(self, size=(500, -1)), (irow, 0), (1, 4), labstyle, 2)

        irow += 1
        sizer.Add(self.label.label,      (irow, 0), (1, 1), labstyle, 2)
        sizer.Add(self.label,       (irow, 1), (1, 1), labstyle, 2)
        sizer.Add(color_label,     (irow, 2), (1, 1), labstyle, 2)
        sizer.Add(self.color,      (irow, 3), (1, 1), labstyle, 2)

        irow += 1
        sizer.Add(toff_label,   (irow, 0), (1, 3), labstyle, 2)

        irow += 1
        sizer.Add(xoff_label,   (irow, 0), (1, 1), labstyle, 2)
        sizer.Add(self.txoff,   (irow, 1), (1, 1), labstyle, 2)
        sizer.Add(yoff_label,   (irow, 2), (1, 1), labstyle, 2)
        sizer.Add(self.tyoff,   (irow, 3), (1, 1), labstyle, 2)

        irow += 1
        sizer.Add(self.show_scalebar,  (irow, 1), (1, 1), labstyle, 2)
        sizer.Add(self.show_label,  (irow, 2), (1, 2), labstyle, 2)

        irow += 1
        sizer.Add(HLine(self, size=(500, -1)), (irow, 0), (1, 4), labstyle, 2)

        autopack(self, sizer)

        self.SetMinSize((500, 350))
        xsiz, ysiz = self.GetBestSize()
        self.SetSize((25*(1 + int(xsiz/25)), 25*(2 + int(ysiz/25))))
        self.Show()
        self.Raise()

    def onGridEvents(self, event=None):
        self.conf.show_grid = self.show_grid.IsChecked()
        self.conf.grid_color = hexcolor(self.grid_color.GetValue()[:3])
        self.conf.grid_alpha = self.grid_alpha.GetValue()
        self.parent.panel.autoset_margins()
        self.parent.panel.redraw()

    def onContourEvents(self, event=None):
        self.conf.ncontour_levels = self.ncontours.GetValue()
        self.conf.contour_labels  = self.showlabels.IsChecked()
        self.parent.onContourToggle()

    def onSliceEvents(self, event=None):
        self.conf.slice_width = self.slice_width.GetValue()
        self.conf.slices = self.slice_dir.GetStringSelection()
        self.conf.slice_onmotion = self.slice_dynamic.IsChecked()
        self.parent.onSliceChoice()

    def onScalebarEvents(self, event=None):
        self.conf.scalebar_show = self.show_scalebar.IsChecked()
        self.conf.scalebar_showlabel = self.show_label.IsChecked()
        self.conf.scalebar_label = self.label.GetValue()
        self.conf.scalebar_pos =  self.ypos.GetValue(), self.xpos.GetValue()
        self.conf.scalebar_textoffset =  self.tyoff.GetValue(), self.txoff.GetValue()
        self.conf.scalebar_size = self.height.GetValue(), self.width.GetValue()

        self.conf.scalebar_color = col = hexcolor(self.color.GetValue()[:3])
        self.conf.scalebar_units  = self.pixunits.GetValue()
        self.conf.scalebar_pixelsize =  self.ypix.GetValue(), self.xpix.GetValue()
        self.parent.panel.redraw()



#!/usr/bin/python
import numpy as np
import wxmplot.interactive as wi

x = np.concatenate((0.25 + np.random.normal(size=2000, scale=0.5),
                   np.random.random(size=600)*1.25,
                   -0.2+np.random.random(size=300)*0.7))

wi.hist(x, bins=51, rwidth=0.75, title='A Histogram')



"""
example showing 2 maps
"""
import wx
from numpy import exp, random, arange, outer, array, linspace
from wxmplot import ImageMatrixFrame

def gauss2d(x, y, x0, y0, sx, sy):
    return outer( exp( -(((y-y0)/float(sy))**2)/2),
                  exp( -(((x-x0)/float(sx))**2)/2) )

if __name__ == '__main__':
    app = wx.App()
    frame = ImageMatrixFrame()
    ny, nx = 41, 51

    x = linspace(0, 1, nx)
    y = linspace(-1, 1, ny)
    ox =  x / 10.0
    oy = -1 + y / 20.0

    map1  = 0.5 * random.random(size=nx*ny).reshape(ny, nx)
    map1 += 0.02 + (16.0*gauss2d(x, y, .50,  .25,   .09,  .08) +
                    18.0*gauss2d(x, y,  .12, -.06,  .23,  .21))
    map2  = 0.3 * random.random(size=nx*ny).reshape(ny, nx)
    map2 += 0.11 + (1.0*gauss2d(x, y,   .44,  .3,   .1,  .15) +
                    1.2*gauss2d(x, y,   .7,  -0.1, .09, .11))

    frame.display(map1, map2, x=x, y=y, name1='Iron', name2='Carbon',
                  xlabel='X', ylabel='Y')
    frame.Show()
    app.MainLoop()



#!/usr/bin/python
##
## MPlot PlotFrame: a wx.Frame for 2D line plotting, using matplotlib
##

import wx
import matplotlib
from functools import partial

from .plotpanel import PlotPanel
from .baseframe import BaseFrame
from .utils import MenuItem

class MultiPlotFrame(BaseFrame):
    """
    MatPlotlib Array of 2D plot as a wx.Frame, using PlotPanel
    """
    default_panelopts = dict(labelfontsize=7, legendfontsize=6)

    def __init__(self, parent=None, rows=1, cols=1, framesize=None,
                 panelsize=(400, 320), panelopts=None, **kws):

        BaseFrame.__init__(self, parent=parent,
                           title  = '2D Plot Array Frame',
                           size=framesize, **kws)
        self.panels = {}
        self.rows = rows
        self.cols = cols
        if framesize is None:
            framesize = (panelsize[0]*cols, panelsize[1]*rows)
        self.framesize = framesize
        self.panelsize = panelsize

        self.panelopts = self.default_panelopts
        if panelopts is not None:
            self.panelopts.update(panelopts)

        self.current_panel = (0, 0)
        self.BuildFrame()

    def set_panel(self,ix,iy):
        self.current_panel = (ix,iy)
        try:
            self.panel = self.panels[(ix,iy)]
        except KeyError:
            print( 'could not set self.panel')

    def plot(self,x,y,panel=None,**kws):
        """plot after clearing current plot """
        if panel is None:
            panel = self.current_panel
        opts = {}
        opts.update(self.default_panelopts)
        opts.update(kws)
        self.panels[panel].plot(x ,y, **opts)

    def oplot(self,x,y,panel=None,**kws):
        """generic plotting method, overplotting any existing plot """
        if panel is None:
            panel = self.current_panel
        opts = {}
        opts.update(self.default_panelopts)
        opts.update(kws)
        self.panels[panel].oplot(x, y, **opts)

    def update_line(self, t, x, y, panel=None, **kw):
        """overwrite data for trace t """
        if panel is None:
            panel = self.current_panel
        self.panels[panel].update_line(t, x, y, **kw)

    def set_xylims(self, lims, axes=None, panel=None):
        """overwrite data for trace t """
        if panel is None: panel = self.current_panel
        self.panels[panel].set_xylims(lims, axes=axes, **kw)

    def clear(self,panel=None):
        """clear plot """
        if panel is None: panel = self.current_panel
        self.panels[panel].clear()

    def unzoom_all(self,event=None,panel=None):
        """zoom out full data range """
        if panel is None: panel = self.current_panel
        self.panels[panel].unzoom_all(event=event)

    def unzoom(self, event=None, panel=None):
        """zoom out 1 level, or to full data range """
        if panel is None:
            panel = self.current_panel
        self.panels[panel].unzoom(event=event)

    def onZoomStyle(self, event=None, style='both x and y'):
        for panel in self.panels.values():
            panel.conf.zoom_style = style

    def set_title(self,s,panel=None):
        "set plot title"
        if panel is None: panel = self.current_panel
        self.panels[panel].set_title(s)

    def set_xlabel(self,s,panel=None):
        "set plot xlabel"
        if panel is None: panel = self.current_panel
        self.panels[panel].set_xlabel(s)

    def set_ylabel(self,s,panel=None):
        "set plot xlabel"
        if panel is None: panel = self.current_panel
        self.panels[panel].set_ylabel(s)

    def save_figure(self,event=None,panel=None):
        """ save figure image to file"""
        if panel is None: panel = self.current_panel
        self.panels[panel].save_figure(event=event)

    def configure(self,event=None,panel=None):
        if panel is None: panel = self.current_panel
        self.panels[panel].configure(event=event)

    ####
    ## create GUI
    ####
    def BuildFrame(self):
        sbar = self.CreateStatusBar(2, wx.CAPTION)
        sfont = sbar.GetFont()
        sfont.SetWeight(wx.BOLD)
        sfont.SetPointSize(10)
        sbar.SetFont(sfont)

        sizer = wx.GridBagSizer(3, 3)

        for i in range(self.rows):
            for j in range(self.cols):
                self.panels[(i,j)] = PlotPanel(self, size=self.panelsize)
                # **self.panelopts)
                self.panels[(i,j)].messenger = self.write_message
                panel = self.panels[(i,j)]

                sizer.Add(panel,(i,j),(1,1),flag=wx.EXPAND|wx.ALIGN_CENTER)
                panel.report_leftdown = partial(self.report_leftdown,
                                                panelkey=(i,j))

        self.panel = self.panels[(0,0)]
        for i in range(self.rows):
            sizer.AddGrowableRow(i)
        for i in range(self.cols):
            sizer.AddGrowableCol(i)


        self.BuildMenu()
        self.SetStatusWidths([-3, -1])
        self.SetStatusText('',0)
        self.SetSize(self.framesize)
        self.SetAutoLayout(True)
        self.SetSizerAndFit(sizer)

    def BuildMenu(self):
        mfile = self.Build_FileMenu()
        mopts = wx.Menu()
        MenuItem(self, mopts, "Configure Plot\tCtrl+K",
                 "Configure Plot styles, colors, labels, etc",
                 self.on_configure)
        MenuItem(self, mopts, "Toggle Legend\tCtrl+L",
                 "Toggle Legend Display",
                 self.on_toggle_legend)
        MenuItem(self, mopts, "Toggle Grid\tCtrl+G",
                 "Toggle Grid Display",
                 self.on_toggle_grid)


        mopts.AppendSeparator()

        MenuItem(self, mopts, "Zoom X and Y\tCtrl+W",
                 "Zoom on both X and Y",
                 partial(self.onZoomStyle, style='both x and y'),
                 kind=wx.ITEM_RADIO, checked=True)
        MenuItem(self, mopts, "Zoom X Only\tCtrl+X",
                 "Zoom X only",
                 partial(self.onZoomStyle, style='x only'),
                 kind=wx.ITEM_RADIO)

        MenuItem(self, mopts, "Zoom Y Only\tCtrl+Y",
                 "Zoom Y only",
                 partial(self.onZoomStyle, style='y only'),
                 kind=wx.ITEM_RADIO)

        MenuItem(self, mopts, "Zoom Out\tCtrl+Z",
                 "Zoom out to full data range",
                 self.unzoom)

        mopts.AppendSeparator()


        mhelp = wx.Menu()
        MenuItem(self, mhelp, "Quick Reference",
                 "Quick Reference for WXMPlot", self.onHelp)
        MenuItem(self, mhelp, "About", "About WXMPlot", self.onAbout)

        mbar = wx.MenuBar()
        mbar.Append(mfile, 'File')
        mbar.Append(mopts, '&Options')
        if self.user_menus is not None:
            for title, menu in self.user_menus:
                mbar.Append(menu, title)

        mbar.Append(mhelp, '&Help')

        self.SetMenuBar(mbar)
        self.Bind(wx.EVT_CLOSE,self.onExit)

    def on_configure(self, event=None, **kws):
        self.panel.configure()

    def on_toggle_legend(self, event=None, **kws):
        self.panel.toggle_legend()

    def on_toggle_grid(self, event=None, **kws):
        self.panel.toggle_grid()

    def on_unzoom(self, event=None, **kws):
        self.panel.unzoom()

    def report_leftdown(self, event=None, panelkey=None,**kw):
        try:
            self.panel.set_bg()
            self.panel.canvas.draw()
        except:
            pass

        if panelkey is None or event is None:
            return

        ix, iy = panelkey
        self.set_panel(ix, iy)
        panel = self.panel
        ex, ey = event.x, event.y
        tmsg = 'Current Panel: (%i, %i) ' % (ix, iy)
        try:
            x, y = panel.axes.transData.inverted().transform((ex, ey))
        except:
            x, y = event.xdata, event.ydata

        try:
            if x is not None and y is not None:
                msg = ("%s X,Y= %s, %s" % (tmsg, panel._xfmt, panel._yfmt)) % (x, y)

            self.write_message(msg, panel=0)
        except:
            pass



#!/usr/bin/env python
# simple wxmplot example

from numpy import linspace, sin, cos, random

def test_import():
    success = False
    try:
        import wxmplot
        success = True
    except:
        pass
    assert(success)



import wx
from .plotframe import PlotFrame

class PlotApp(object):
    """wx.App that provides a top-level Frame containing a
    matpplotlib plot that includes (by default) zooming,
    navigation and user-customization of line colors, width,
    labels, and so on.
      arguments
      ---------
      title    frame title      "WXMPLOT"
      size     wx Size tuple    (700, 450)
      dpi      Figure dpi       150

    Additional keyword arguments will be passed to PlotPanel
    """
    def __init__(self, title='WXMPLOT', size=(700,450),  dpi=150, **kws):
        self.app   = wx.App()
        self.frame = PlotFrame(title=title, size=size, dpi=dpi, **kws)

    def plot(self, x, y, **kw):
        """plot x, y values (erasing old plot),
        for method options see PlotPanel.plot.
        """
        return self.frame.plot(x,y,**kw)

    def oplot(self, x, y, **kw):
        """overplot x, y values (on top of old plot),
        for method options see PlotPanel.oplot
        """
        return self.frame.oplot(x,y,**kw)

    def set_title(self,s):
        "set title"
        self.frame.set_title(s)

    def write_message(self, msg, **kw):
        "write message to status bar"
        return self.frame.write_message(msg, **kw)

    def draw(self):
        "update figure"
        self.frame.draW()

    def run(self):
        self.frame.Show()
        self.frame.Raise()
        self.app.MainLoop()



from wxmplot import PlotApp
from numpy import arange, sin, cos, exp, pi

xx  = arange(0.0,12.0,0.1)
y1  = 1*sin(2*pi*xx/3.0)
y2  = 4*cos(2*pi*(xx-1)/5.0)/(6+xx)
y3  = -pi + 2*(xx/10. + exp(-(xx-3)/5.0))

p = PlotApp()
p.plot(xx, y1, color='blue',  style='dashed',
       title='Example PlotApp',  label='a',
       ylabel=r'$k^2\chi(k) $',
       xlabel=r'$  k \ (\AA^{-1}) $' )

p.oplot(xx, y2,  marker='+', linewidth=0, label =r'$ x_1 $')
p.oplot(xx, y3,  style='solid',          label ='x_2')
p.write_message('Try Help->Quick Reference')
p.run()



#!/usr/bin/env python
from numpy import linspace, sin, random
import wxmplot.interactive as wi

x = linspace(0.0, 15.0, 151)
y = 5*sin(4*x)/(x*x+6)
z = 4.8*sin(4.3*x)/(x*x+8) + random.normal(size=len(x), scale=0.05)

for ix, theme in enumerate(('light', 'dark', 'matplotlib', 'seaborn',
                            'ggplot', 'bmh', 'tableau-colorblind10')):
    this = wi.plot(x, y, wintitle='%s theme' % theme,   win=ix+1,
                   title=theme,
                   ylabel=r'$\phi(\tau)$', xlabel=r'$\tau \> \rm (ms)$',
                   label='reference', show_legend=True, theme=theme, size=(600,450))
    wi.plot(x, z, label='signal', marker='+', win=ix+1)
    siz = this.GetSize()
    this.SetPosition((25 + int(ix*siz[0]/4), 25 + int(ix*siz[1]/12)))



#!/usr/bin/python
##
## MPlot BasePanel: a Basic Panel for 2D line and image plotting
##

import sys
import time
import os
from math import log10

import wx
from wxutils import get_cwd
import numpy as np
import matplotlib
from matplotlib.widgets import Lasso
from matplotlib import dates
from matplotlib.backends.backend_wx import RendererWx

from .utils import Printer, MenuItem

class BasePanel(wx.Panel):
    """
    wx.Panel component shared by PlotPanel and ImagePanel.

    provides:
         Basic support Zooming / Unzooming
         support for Printing
         popup menu
         bindings for keyboard short-cuts
    """
    def __init__(self, parent, messenger=None,
                 show_config_popup=True, zoom_callback=None,
                 output_title=None, **kws):

        wx.Panel.__init__(self, parent, -1, **kws)

        self.is_macosx = (os.name == 'posix' and
                          sys.platform.lower().startswith('darw'))

        self.messenger = messenger
        if messenger is None:
            self.messenger = self.__def_messenger

        self.popup_menu =  None
        self._yfmt  = self._y2fmt = self._xfmt  = None
        self.use_dates = False
        self.show_config_popup = show_config_popup
        self.launch_dir  = get_cwd()

        self.mouse_uptime = time.time()
        self.user_limits = {}
        self.zoom_ini  = None  # x, y coords for zoom-box
        self.zoom_callback = zoom_callback
        self.rbbox = None
        # self.zdc = None
        self.cursor_modes = {}
        self.cursor_mode = 'report'
        self.parent = parent
        self.motion_sbar = None
        self.printer = Printer(self, title=output_title)
        self.add_cursor_mode('report', motion = self.report_motion,
                             leftdown = self.report_leftdown)
        self.add_cursor_mode('zoom', motion = self.zoom_motion,
                             leftdown = self.zoom_leftdown,
                             leftup   = self.zoom_leftup)
        self.add_cursor_mode('lasso', motion = self.lasso_motion,
                             leftdown = self.lasso_leftdown,
                             leftup   = self.lasso_leftup)


    def addCanvasEvents(self):
        # use matplotlib events
        self.canvas.mpl_connect("motion_notify_event",
                                self.__onMouseMotionEvent)
        self.canvas.mpl_connect("button_press_event",
                                self.__onMouseButtonEvent)
        self.canvas.mpl_connect("button_release_event",
                                self.__onMouseButtonEvent)
        self.canvas.mpl_connect("key_press_event",
                                self.__onKeyEvent)

        if os.name == 'posix':
            def swallow_mouse(*args):
                pass
            self.canvas.CaptureMouse = swallow_mouse
            self.canvas.ReleaseeMouse = swallow_mouse

        self.BuildPopup()

    def BuildPopup(self):
        # build pop-up menu for right-click display
        self.popup_menu = popup = wx.Menu()
        MenuItem(self, popup, 'Undo last zoom', '',   self.unzoom)
        MenuItem(self, popup, 'Zoom all the way out', '',   self.unzoom_all)

        if self.show_config_popup:
            MenuItem(self, popup, 'Configure', '',   self.configure)

        MenuItem(self, popup, 'Save Image', '',   self.save_figure)


    def clear(self):
        """ clear plot """
        self.axes.cla()
        self.conf.ntrace = 0
        self.conf.xlabel = ''
        self.conf.ylabel = ''
        self.conf.title  = ''

    def unzoom_all(self, event=None):
        """ zoom out full data range """
        if len(self.conf.zoom_lims) > 0:
            self.conf.zoom_lims = [self.conf.zoom_lims[0]]
        self.user_lims[self.axes] = 4*[None]
        self.unzoom(event)

    def unzoom(self, event=None, set_bounds=True):
        """ zoom out 1 level, or to full data range """
        lims = None
        if len(self.conf.zoom_lims) > 1:
            lims = self.conf.zoom_lims.pop()
        ax = self.axes
        if lims is None: # auto scale
            self.conf.zoom_lims = [None]
            xmin, xmax, ymin, ymax = self.data_range
            ax.set_xlim((xmin, xmax), emit=True)
            ax.set_ylim((ymin, ymax), emit=True)
            if set_bounds:
                ax.update_datalim(((xmin, ymin), (xmax, ymax)))
                ax.set_xbound(ax.xaxis.get_major_locator(
                    ).view_limits(xmin, xmax))
                ax.set_ybound(ax.yaxis.get_major_locator(
                    ).view_limits(ymin, ymax))
        else:
            self.set_viewlimits()

        self.canvas.draw()

    def get_right_axes(self):
        "create, if needed, and return right-hand y axes"
        if len(self.fig.get_axes()) < 2:
            ax = self.axes.twinx()

        return self.fig.get_axes()[1]

    def set_xylims(self, limits, axes=None):
        if axes not in self.user_limits:
            axes = self.axes
        self.user_limits[axes] = limits
        self.unzoom_all()

    def get_viewlimits(self, axes=None):
        if axes is None: axes = self.axes
        xmin, xmax = axes.get_xlim()
        ymin, ymax = axes.get_ylim()
        return (xmin, xmax, ymin, ymax)

    def set_title(self, s, delay_draw=False):
        "set plot title"
        self.conf.relabel(title=s, delay_draw=delay_draw)

    def set_xlabel(self, s, delay_draw=False):
        "set plot xlabel"
        self.conf.relabel(xlabel=s, delay_draw=delay_draw)

    def set_ylabel(self, s, delay_draw=False):
        "set plot ylabel"
        self.conf.relabel(ylabel=s, delay_draw=delay_draw)

    def set_y2label(self, s, delay_draw=False):
        "set plot ylabel"
        self.conf.relabel(y2label=s, delay_draw=delay_draw)

    def write_message(self, s, panel=0):
        """ write message to message handler
        (possibly going to GUI statusbar)"""
        self.messenger(s, panel=panel)

    def save_figure(self, event=None, transparent=False, dpi=600):
        """ save figure image to file"""
        file_choices = "PNG (*.png)|*.png|SVG (*.svg)|*.svg|PDF (*.pdf)|*.pdf"
        try:
            ofile = self.conf.title.strip()
        except:
            ofile = 'Image'
        if len(ofile) > 64:
            ofile = ofile[:63].strip()
        if len(ofile) < 1:
            ofile = 'plot'

        for c in ' :";|/\\': # "
            ofile = ofile.replace(c, '_')

        ofile = ofile + '.png'
        orig_dir = os.path.abspath(get_cwd())
        dlg = wx.FileDialog(self, message='Save Plot Figure as...',
                            defaultDir=orig_dir,
                            defaultFile=ofile,
                            wildcard=file_choices,
                            style=wx.FD_SAVE|wx.FD_CHANGE_DIR)

        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            if hasattr(self, 'fig'):
                self.fig.savefig(path, transparent=transparent, dpi=dpi)
            else:
                self.canvas.print_figure(path, transparent=transparent, dpi=dpi)
            if (path.find(self.launch_dir) ==  0):
                path = path[len(self.launch_dir)+1:]
            self.write_message('Saved plot to %s' % path)
        os.chdir(orig_dir)

    def set_bg(self, color= None):
        if color is None:
            color = '#FBFBFB'
        self.fig.set_facecolor(color)

    def configure(self, evt=None, **kws):
        pass

    ## cursor modes:
    def add_cursor_mode(self, modename, leftdown=None, leftup=None,
                        rightdown=None, rightup=None, motion=None,
                        keyevent=None):

        d = {'leftdown':  leftdown,     'leftup':    leftup,
             'rightdown': rightdown,    'rightup':   rightup,
             'motion':    motion,       'keyevent':  keyevent}
        self.cursor_modes[modename]  = d

    def cursor_mode_action(self,  eventname, **kws):
        mode = self.cursor_mode
        if mode not in self.cursor_modes:
            return
        if hasattr(self.cursor_modes[mode][eventname], '__call__'):
            self.cursor_modes[mode][eventname](**kws)

    ####
    ## GUI events
    ####
    def toggle_legend(self, evt=None, show=None):
        pass

    def toggle_grid(self, evt=None, show=None):
        pass

    def lassoHandler(self, vertices):
        try:
            del self.lasso
            self.canvas.draw_idle()
        except:
            pass

    def zoom_OK(self, start, stop):
        return True

    def report_leftdown(self, event=None):
        if event is None:
            return
        self.write_message("%g, %g" % (event.xdata, event.ydata), panel=1)

    def onLeftDown(self, event=None):
        """ left button down: report x,y coords, start zooming mode"""
        if event is None:
            return
        self.cursor_mode_action('leftdown', event=event)
        self.ForwardEvent(event=event.guiEvent)

    def onLeftUp(self, event=None):
        """ left button up"""
        if event is None:
            return
        self.cursor_mode_action('leftup', event=event)
        self.canvas.draw_idle()
        self.canvas.draw()
        self.ForwardEvent(event=event.guiEvent)

    def ForwardEvent(self, event=None):
        """finish wx event, forward it to other wx objects"""
        if event is not None:
            event.Skip()
            if self.HasCapture():
                try:
                    self.ReleaseMouse()
                except:
                    pass

    def onRightDown(self, event=None):
        """ right button down: show pop-up"""
        if event is None:
            return
        # note that the matplotlib event location have to be converted
        if event.inaxes is not None and self.popup_menu is not None:
            pos = event.guiEvent.GetPosition()
            wx.CallAfter(self.PopupMenu, self.popup_menu, pos)
        self.cursor_mode_action('rightdown', event=event)
        self.ForwardEvent(event=event.guiEvent)

    def onRightUp(self, event=None):
        """ right button up: put back to cursor mode"""
        if event is None:
            return
        self.cursor_mode_action('rightup', event=event)
        self.ForwardEvent(event=event.guiEvent)

    ####
    ## private methods
    ####
    def __def_messenger(self, s, panel=0):
        """ default, generic messenger: write to stdout"""
        sys.stdout.write(s)

    def __date_format(self, x):
        """ formatter for date x-data. primitive, and probably needs
        improvement, following matplotlib's date methods.
        """

        if x < 1: x = 1

        span = self.axes.xaxis.get_view_interval()
        tmin = max(1.0, span[0])
        tmax = max(2.0, span[1])
        tmin = time.mktime(dates.num2date(tmin).timetuple())
        tmax = time.mktime(dates.num2date(tmax).timetuple())
        nhours = (tmax - tmin)/3600.0
        fmt = "%m/%d"
        if nhours < 0.1:
            fmt = "%H:%M\n%Ssec"
        elif nhours < 4:
            fmt = "%m/%d\n%H:%M"
        elif nhours < 24*8:
            fmt = "%m/%d\n%H:%M"
        try:
            return time.strftime(fmt, dates.num2date(x).timetuple())
        except:
            return "?"

    def reset_formats(self):
        self._xfmt = self._yfmt = self._y2fmt = None

    def xformatter(self, x, pos):
        " x-axis formatter "
        if self.use_dates:
            return self.__date_format(x)
        else:
            return self.__format(x, type='x')

    def yformatter(self, y, pos):
        " y-axis formatter "
        return self.__format(y, type='y')

    def y2formatter(self, y, pos):
        " y-axis formatter "
        return self.__format(y, type='y2')

    def set_format_str(self, axis):
        try:
            ticks = axis.get_major_locator()()
        except:
            ticks = [0, 1]

        if len(ticks) < 2:
            ticks.append(0)
            ticks.append(1)

        step = max(2.e-15, abs(np.diff(ticks).mean()))
        if step > 5e4 or (step < 5.e-4 and ticks.mean() < 5.e-2):
            fmt = '%.2e'
        else:
            ndigs = max(0, 3 - round(log10(step)))
            while ndigs >= 0:
                if np.abs(ticks- np.round(ticks, decimals=ndigs)).max() < 2e-3*step:
                    ndigs -= 1
                else:
                    break
            fmt = '%%1.%df' % min(9, ndigs+1)
        return fmt


    def __format(self, x, type='x'):
        """ home built tick formatter to use with FuncFormatter():
        x     value to be formatted
        type  'x' or 'y' or 'y2' to set which list of ticks to get

        also sets self._yfmt/self._xfmt for statusbar
        """
        fmt, v = '%1.5g','%1.5g'
        if type == 'y':
            ax = self.axes.yaxis
            if self._yfmt is None:
                self._yfmt = self.set_format_str(ax)
            fmt = self._yfmt
        elif type == 'y2' and len(self.fig.get_axes()) > 1:
            ax =  self.fig.get_axes()[1].yaxis
            if self._y2fmt is None:
                self._y2fmt = self.set_format_str(ax)
            fmt = self._y2fmt
        else:
            ax = self.axes.xaxis
            if self._xfmt is None:
                self._xfmt = self.set_format_str(ax)
            fmt = self._xfmt

        s =  fmt % x
        s.strip()
        s = s.replace('+', '')
        while s.find('e0')>0:
            s = s.replace('e0','e')
        if s.endswith('e'):
            s = s[:-1]
        while s.find('-0')>0:
             s = s.replace('-0','-')
        return s

    def __onKeyEvent(self, event=None):
        """ handles key events on canvas
        """
        if event is None:
            return
        key = event.guiEvent.GetKeyCode()
        shift = event.guiEvent.ShiftDown()
        mod  = event.guiEvent.ControlDown()
        if self.is_macosx:
            mod = event.guiEvent.MetaDown()
        if mod:
            if (key < wx.WXK_SPACE or  key > 255):
                return
            ckey = chr(key)
            if ckey == 'C':
                self.canvas.Copy_to_Clipboard(event)
            elif ckey == 'S':
                self.save_figure(event)
            elif ckey == 'K':
                self.configure(event)
            elif ckey == 'Z':
                self.unzoom(event)
            elif ckey == 'P':
                self.canvas.printer.Print(event)
            elif ckey == 'X':
                self.conf.zoom_style = 'x only'
            elif ckey == 'Y':
                self.conf.zoom_style = 'y only'
            elif ckey == 'W':
                self.conf.zoom_style = 'both x and y'
        elif key in (wx.WXK_LEFT, wx.WXK_NUMPAD_LEFT):
            self._onPan(direction='left', shift=shift)
        elif key in (wx.WXK_RIGHT, wx.WXK_NUMPAD_RIGHT):
            self._onPan(direction='right', shift=shift)
        elif key in (wx.WXK_UP, wx.WXK_NUMPAD_UP):
            self._onPan(direction='up', shift=shift)
        elif key in (wx.WXK_DOWN, wx.WXK_NUMPAD_DOWN):
            self._onPan(direction='down', shift=shift)

    def _onPan(self, direction=None, shift=False):
        if direction not in ('left', 'right', 'up', 'down'):
            return

        axes = self.fig.get_axes()[0]
        try:
            if len(self.conf.zoom_lims) > 0:
                x0, x1, y0, y1 = self.conf.zoom_lims[-1][axes]
            else:
                x0, x1 = axes.get_xlim()
                y0, y1 = axes.get_ylim()
        except:
            return

        step = 0.10 if shift else 0.02
        if direction in ('left', 'right'):
            step *= abs(x1 - x0) * {'right': 1, 'left': -1}[direction]
            x0, x1 = x0+step, x1+step
        elif direction in ('up', 'down'):
            step *= abs(y1 - y0) * {'up': 1, 'down': -1}[direction]
            y0, y1 = y0+step, y1+step

        self.conf.zoom_lims.append({axes: [x0, x1, y0, y1]})
        self.set_viewlimits()
        self.canvas.draw()

    def __onMouseButtonEvent(self, event=None):
        """ general mouse press/release events. Here, event is
        a MplEvent from matplotlib.  This routine just dispatches
        to the appropriate onLeftDown, onLeftUp, onRightDown, onRightUp....
        methods.
        """
        if event is None:
            return
        button = event.button or 1

        handlers = {(1, 'button_press_event'):   self.onLeftDown,
                    (1, 'button_release_event'): self.onLeftUp,
                    (3, 'button_press_event'):   self.onRightDown}

        handle_event = handlers.get((button, event.name), None)
        if hasattr(handle_event, '__call__'):
            handle_event(event)
        event.guiEvent.Skip()


    def __onMouseMotionEvent(self, event=None):
        """Draw a cursor over the axes"""
        if event is None:
            return
        self.cursor_mode_action('motion', event=event)


    def gui_repaint(self, drawDC=None):
        """
        Update the displayed image on the GUI canvas, using the supplied
        wx.PaintDC device context.

        The 'WXAgg' backend sets origin accordingly.
        """
        if not drawDC:
            drawDC = wx.ClientDC(self.canvas)

        bmp = self.canvas.bitmap
        if (wx.Platform == '__WXMSW__'
            and isinstance(self.canvas.figure._cachedRenderer, RendererWx)):
            bmp = bmp.ConvertToImage().ConvertToBitmap()

        drawDC.DrawBitmap(bmp, 0, 0)
        if self.rbbox is not None:
            drawDC.SetLogicalFunction(wx.XOR)
            drawDC.SetBrush(wx.Brush('Black', wx.BRUSHSTYLE_TRANSPARENT))
            drawDC.SetPen(wx.Pen('WHITE', 2, wx.SOLID))
            drawDC.DrawRectangle(*self.rbbox)

    def zoom_motion(self, event=None):
        """motion event handler for zoom mode"""
        try:
            x, y  = event.x, event.y
        except:
            return
        self.report_motion(event=event)
        if self.zoom_ini is None:
            return
        ini_x, ini_y, ini_xd, ini_yd = self.zoom_ini
        if event.xdata is not None:
            self.x_lastmove = event.xdata
        if event.ydata is not None:
            self.y_lastmove = event.ydata
        x0     = min(x, ini_x)
        ymax   = max(y, ini_y)
        width  = abs(x-ini_x)
        height = abs(y-ini_y)
        y0     = self.canvas.figure.bbox.height - ymax
        limits = self.canvas.figure.axes[0].bbox.corners()
        if self.conf.zoom_style.startswith('x'):
            height = int(round(limits[3][1] - limits[0][1]))
            y0 = 1 + self.canvas.GetSize()[1] - int(round(limits[1][1]))

        elif self.conf.zoom_style.startswith('y'):
            width = 1 + int(round(limits[2][0] - limits[0][0]))
            x0 = 1 + int(round(limits[0][0]))

        self.rbbox = (x0, y0, width, height)
        self.canvas.Refresh()


    def zoom_leftdown(self, event=None):
        """leftdown event handler for zoom mode"""
        self.x_lastmove, self.y_lastmove = None, None
        self.zoom_ini = (event.x, event.y, event.xdata, event.ydata)
        self.report_leftdown(event=event)

    def zoom_leftup(self, event=None):
        """leftup event handler for zoom mode"""
        if self.zoom_ini is None:
            return
        self.canvas._rubberband_rect = None

        ini_x, ini_y, ini_xd, ini_yd = self.zoom_ini
        try:
            dx = abs(ini_x - event.x)
            dy = abs(ini_y - event.y)
        except:
            dx, dy = 0, 0
        t0 = time.time()
        self.rbbox = None
        self.zoom_ini = None
        if (dx > 3) and (dy > 3) and (t0-self.mouse_uptime)>0.1:
            self.mouse_uptime = t0
            zlims, tlims = {}, {}
            for ax in self.fig.get_axes():
                xmin, xmax = ax.get_xlim()
                ymin, ymax = ax.get_ylim()
                zlims[ax] = [xmin, xmax, ymin, ymax]
            if len(self.conf.zoom_lims) == 0:
                self.conf.zoom_lims.append(zlims)
            # for multiple axes, we first collect all the new limits, and
            # only then apply them
            for ax in self.fig.get_axes():
                ax_inv = ax.transData.inverted
                try:
                    x1, y1 = ax_inv().transform((event.x, event.y))
                except:
                    x1, y1 = self.x_lastmove, self.y_lastmove
                try:
                    x0, y0 = ax_inv().transform((ini_x, ini_y))
                except:
                    x0, y0 = ini_xd, ini_yd

                tlims[ax] = [min(x0, x1), max(x0, x1),
                             min(y0, y1), max(y0, y1)]
                if self.conf.zoom_style.startswith('x'):
                    tlims[ax][2:] = [ymin, ymax]
                elif self.conf.zoom_style.startswith('y'):
                    tlims[ax][:2] = [xmin, xmax]

            self.conf.zoom_lims.append(tlims)
            # now apply limits:
            self.set_viewlimits()

            if callable(self.zoom_callback):
                self.zoom_callback(wid=self.GetId(), limits=tlims[ax])


    def lasso_motion(self, event=None):
        """motion event handler for lasso mode"""
        self.report_motion(event=event)

    def lasso_leftdown(self, event=None):
        """leftdown event handler for lasso mode"""
        try:
            self.report_leftdown(event=event)
        except:
            return

        if event.inaxes:
            # set lasso color
            color='goldenrod'
            cmap = getattr(self.conf, 'cmap', None)
            if isinstance(cmap, dict):
                cmap = cmap['int']
            try:
                if cmap is not None:
                    rgb = (int(i*255)^255 for i in cmap._lut[0][:3])
                    color = '#%02x%02x%02x' % tuple(rgb)
            except:
                pass
            self.lasso = Lasso(event.inaxes, (event.xdata, event.ydata),
                               self.lassoHandler)
            self.lasso.line.set_color(color)

    def lasso_leftup(self, event=None):
        """leftup event handler for lasso mode"""
        pass

    def report_motion(self, event=None):
        if event.inaxes is None:
            return

        fmt = "X,Y= %g, %g"
        x, y  = event.xdata, event.ydata
        if len(self.fig.get_axes()) > 1:
            try:
                x, y = self.axes.transData.inverted().transform((event.x, event.y))
            except:
                pass
        if self.motion_sbar is None:
            try:
                self.motion_sbar = self.nstatusbar-1
            except AttributeError:
                self.motion_sbar = 1

        self.write_message(fmt % (x, y), panel=self.motion_sbar)

    def Print(self, event=None, **kw):
        self.printer.Print(event=event, **kw)

    def PrintPreview(self, event=None, **kw):
        self.printer.Preview(event=event, **kw)

    def PrintSetup(self, event=None, **kw):
        self.printer.Setup(event=event, **kw)



# wxmplot documentation build configuration file
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import os
import sys
from datetime import date

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.append(os.path.abspath(os.path.join('.')))

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.autodoc', 'sphinx.ext.mathjax', 'sphinx.ext.napoleon']

autoclass_content = 'both'

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = {'.rst': 'restructuredtext'}

# The encoding of source files.
#source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'wxmplot'
copyright = u'{}, Matthew Newville, The University of Chicago'.format(date.today().year)

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
try:
    import wxmplot
    release = wxmplot.__version__.split('+', 1)[0]
except ImportError:
    release = '0.1'


# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
#today_fmt = '%B %d, %Y'

# List of documents that shouldn't be included in the build.
#unused_docs = []

# List of directories, relative to source directory, that shouldn't be searched
# for source files.
exclude_trees = ['_build']

# The reST default role (used for this markup: `text`) to use for all documents.
default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = False

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
#modindex_common_prefix = []


# -- Options for HTML output ---------------------------------------------------

# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
html_theme_path = ['sphinx_theme']
html_theme = 'sphinxdoc'


html_title = "WXMPLOT: plotting widgets using wxPython and matplotlib"
html_short_title = "WXMPLOT: wxPython+matplotlib"


#html_logo = None
# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
#html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
#html_last_updated_fmt = '%b %d, %Y'

# Custom sidebar templates, maps document names to template names.
html_sidebars = {'index': ['indexsidebar.html', 'searchbox.html']}

html_domain_indices = False
html_use_index = True
#html_split_index = False

# If true, links to the reST sources are added to the pages.
html_show_sourcelink = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# If nonempty, this is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = ''

# Output file base name for HTML help builder.
htmlhelp_basename = 'wxmplotdoc'


# -- Options for LaTeX output --------------------------------------------------

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual]).
latex_documents = [
  ('index', 'wxmplot.tex', u'wxmplot documentation',
   u'Matthew Newville', 'manual'),
]

# # configuration for jupyter_sphinx
# package_path = os.path.abspath('../..')
# os.environ['PYTHONPATH'] = ':'.join((package_path, os.environ.get('PYTHONPATH', '')))
#
# image_converter_args=["-density", "300"]
#
# # Sphinx-gallery configuration
# sphinx_gallery_conf = {
#     'examples_dirs': '../examples',
#     'gallery_dirs': 'examples',
#     'filename_pattern': '/documentation|/example_',
#     'ignore_pattern': '/doc_',
#     'expected_failing_examples': ['../examples/documentation/model_loadmodel.py']
# }



#!/usr/bin/python
##
## MPlot PlotPanel: a wx.Panel for 2D line plotting, using matplotlib
##

import sys
import time
import os
import wx
from threading import Thread

import numpy as np
import matplotlib
import matplotlib.cm as cmap
from matplotlib.figure import Figure
from matplotlib.gridspec import GridSpec
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg
from matplotlib.patches import Rectangle

from .imageconf import ImageConfig, RGB_COLORS
from .basepanel import BasePanel
from .utils import inside_poly, MenuItem
from .plotframe import PlotFrame

class ImagePanel(BasePanel):
    """
    MatPlotlib Image as a wx.Panel, suitable for embedding
    in any wx.Frame.   This provides a right-click popup
    menu for configuration, zoom by dragging, saving an image
    figure, and Ctrl-C for copy-image-to-clipboard, customizations
    of colormap, interpolation, and intesity scaling

    For more features, see PlotFrame, which embeds a PlotPanel
    and also provides, a Menu, StatusBar, and Printing support.
    """

    def __init__(self, parent, messenger=None, data_callback=None,
                 cursor_callback=None, lasso_callback=None,
                 redraw_callback=None, zoom_callback=None,
                 contour_callback=None, size=(525, 450), dpi=100,
                 output_title='Image', **kws):

        matplotlib.rc('lines', linewidth=2)
        BasePanel.__init__(self, parent,
                           output_title=output_title,
                           messenger=messenger,
                           zoom_callback=zoom_callback, **kws)

        self.conf = ImageConfig()
        self.conf.title = output_title
        self.cursor_mode = 'zoom'
        self.data_callback = data_callback
        self.cursor_callback = cursor_callback
        self.lasso_callback = lasso_callback
        self.contour_callback = contour_callback
        self.redraw_callback = redraw_callback
        self.slice_plotframe = None
        self.win_config = None
        self.size = size
        self.dpi  = dpi
        self.user_limits = {}
        self.scalebar_rect = self.scalerbar_text = None
        self.BuildPanel()

    @property
    def xdata(self):
        return self.conf.xdata

    @xdata.setter
    def xdata(self, value):
        self.conf.xdata = value

    @property
    def ydata(self):
        return self.conf.ydata

    @ydata.setter
    def ydata(self, value):
        self.conf.ydata = value


    def display(self, data, x=None, y=None, xlabel=None, ylabel=None,
                style=None, nlevels=None, levels=None, contour_labels=None,
                store_data=True, col=0, unzoom=True, show_axis=False,
                auto_contrast=False, contrast_level=0, colormap=None, **kws):
        """
        generic display, using imshow (default) or contour
        """
        if style is not None:
            self.conf.style = style
        self.axes.cla()
        conf = self.conf
        conf.log_scale = False
        conf.show_axis = show_axis
        conf.highlight_areas = []
        if 1 in data.shape:
            data = data.squeeze()
        self.data_range = [0, data.shape[1], 0, data.shape[0]]
        if contrast_level not in (0, None):
            conf.contrast_level = contrast_level
        if auto_contrast:
            conf.contrast_level = 1
        if x is not None:
            conf.xdata = np.array(x)
            if conf.xdata.shape[0] != data.shape[1]:
                conf.xdata = None
        if y is not None:
            conf.ydata = np.array(y)
            if conf.ydata.shape[0] != data.shape[0]:
                conf.ydata = None

        if xlabel is not None:
            conf.xlab = xlabel
        if ylabel is not None:
            conf.ylab = ylabel
        if store_data:
            conf.data = data

        if self.conf.style == 'contour':
            if levels is None:
                levels = self.conf.ncontour_levels
            else:
                self.conf.ncontour_levels = levels
            if nlevels is None:
                nlevels = self.conf.ncontour_levels = 9
            nlevels = max(2, nlevels)

            if conf.contrast_level is not None:
                contrast = [conf.contrast_level, 100.0-conf.contrast_level]
                imin, imax = np.percentile(conf.data, contrast)
                data = np.clip(conf.data, imin, imax)

            clevels  = np.linspace(data.min(), data.max(), nlevels+1)
            self.conf.contour_levels = clevels
            self.conf.image = self.axes.contourf(data, cmap=self.conf.cmap[col],
                                                 levels=clevels)

            self.conf.contour = self.axes.contour(data, cmap=self.conf.cmap[col],
                                                  levels=clevels)
            cmap_name = self.conf.cmap[col].name
            xname = 'gray'
            try:
                if cmap_name == 'gray_r':
                    xname = 'Reds_r'
                elif cmap_name == 'gray':
                    xname = 'Reds'
                elif cmap_name.endswith('_r'):
                    xname = 'gray_r'
            except:
                pass
            self.conf.contour.set_cmap(getattr(cmap, xname))

            if contour_labels is None:
                contour_labels = self.conf.contour_labels
            if contour_labels:
                nlog = np.log10(abs(clevels[1]-clevels[0]))
                fmt = "%.4f"
                if nlog < -2:
                    fmt = "%%.%df" % (1-nlog)
                elif nlog > 2:
                    fmt = "%.1f"
                self.axes.clabel(self.conf.contour, fontsize=10, inline=1, fmt=fmt)
            if hasattr(self.contour_callback , '__call__'):
                self.contour_callback(levels=clevels)
        else:
            if data.max() == data.min():
                img = data
            else:
                img = (data - data.min()) /(1.0*data.max() - data.min())
            if colormap is not None:
                self.conf.set_colormap(colormap, icol=col)
            self.conf.image = self.axes.imshow(img, cmap=self.conf.cmap[col],
                                               interpolation=self.conf.interp)

        self.autoset_margins()

        if unzoom:
            self.unzoom_all()

        if hasattr(self.data_callback, '__call__'):
            self.data_callback(data, x=x, y=y, **kws)

        self.conf.indices = None
        self.indices_thread = Thread(target=self.calc_indices, args=(data.shape, ))
        self.indices_thread.start()


    def update_image(self, data):
        """
        update image on panel, as quickly as possible
        """
        if 1 in data.shape:
            data = data.squeeze()
        if self.conf.contrast_level is not None:
            clevels = [self.conf.contrast_level, 100.0-self.conf.contrast_level]
            imin, imax = np.percentile(data, clevels)
            data = np.clip((data - imin)/(imax - imin + 1.e-8), 0, 1)
        self.axes.images[0].set_data(data)
        self.canvas.draw()


    def autoset_margins(self):
        """auto-set margins  left, bottom, right, top
        according to the specified margins (in pixels)
        and axes extent (taking into account labels,
        title, axis)
        """
        if self.conf.show_axis:
            self.axes.set_axis_on()
            if self.conf.show_grid:
                self.axes.grid(True,
                               alpha=self.conf.grid_alpha,
                               color=self.conf.grid_color)
            else:
                self.axes.grid(False)
            self.conf.set_formatters()

            l, t, r, b = 0.08, 0.96, 0.96, 0.08
            if self.conf.xlab is not None:
                self.axes.set_xlabel(self.conf.xlab)
                b, t = 0.11, 0.96
            if self.conf.ylab is not None:
                self.axes.set_ylabel(self.conf.ylab)
                l, r = 0.11, 0.96
        else:
            self.axes.set_axis_off()
            l, t, r, b = 0.01, 0.99, 0.99, 0.01
        self.gridspec.update(left=l, top=t, right=r, bottom=b)

        for ax in self.fig.get_axes():
            # ax.update_params()
            # ax.set_position(ax.figbox)
            figpos = ax.get_subplotspec().get_position(self.canvas.figure)
            ax.set_position(figpos)

    def add_highlight_area(self, mask, label=None, col=0):
        """add a highlighted area -- outline an arbitrarily shape --
        as if drawn from a Lasso event.
        This takes a mask, which should be a boolean array of the
        same shape as the image.
        """
        patch = mask * np.ones(mask.shape) * 0.9
        cmap = self.conf.cmap[col]
        area = self.axes.contour(patch, cmap=cmap, levels=[0, 1])
        self.conf.highlight_areas.append(area)
        if not hasattr(cmap, '_lut'):
            try:
                cmap._init()
            except:
                pass

        if hasattr(cmap, '_lut'):
            rgb  = [int(i*240)^255 for i in cmap._lut[0][:3]]
            col  = '#%02x%02x%02x' % (rgb[0], rgb[1], rgb[2])
        if label is not None:
            def fmt(*args, **kws): return label
            self.axes.clabel(area, fontsize=9, fmt=fmt,
                             colors=col, rightside_up=True)

        if col is not None:
            for l in area.collections:
                l.set_edgecolor(col)
        self.canvas.draw()

    def set_viewlimits(self, axes=None):
        """ update xy limits of a plot"""
        if axes is None:
            axes = self.axes

        xmin, xmax, ymin, ymax = self.data_range
        if len(self.conf.zoom_lims) >1:
            zlims = self.conf.zoom_lims[-1]
            if axes in zlims:
                xmin, xmax, ymin, ymax = zlims[axes]

        xmin = max(self.data_range[0], xmin)
        xmax = min(self.data_range[1], xmax)
        ymin = max(self.data_range[2], ymin)
        ymax = min(self.data_range[3], ymax)
        if (xmax < self.data_range[0] or
            xmin > self.data_range[1] or
            ymax < self.data_range[2] or
            ymin > self.data_range[3] ):
            self.conf.zoom_lims.pop()
            return

        if abs(xmax-xmin) < 2:
            xmin = int(0.5*(xmax+xmin) - 1)
            xmax = xmin + 2

        if abs(ymax-ymin) < 2:
            ymin = int(0.5*(ymax+xmin) - 1)
            ymax = ymin + 2

        self.axes.set_xlim((xmin, xmax),emit=True)
        self.axes.set_ylim((ymin, ymax),emit=True)
        self.axes.update_datalim(((xmin, ymin), (xmax, ymax)))

        self.conf.datalimits = [xmin, xmax, ymin, ymax]
        self.conf.reset_formats()
        self.redraw()

    def clear(self):
        """ clear plot """
        self.axes.cla()
        self.conf.title  = ''


    ####
    ## create GUI
    ####
    def BuildPanel(self):
        """ builds basic GUI panel and popup menu"""
        figsize = (1.0*self.size[0]/self.dpi, 1.0*self.size[1]/self.dpi)
        self.fig   = Figure(figsize, dpi=self.dpi)
        self.gridspec = GridSpec(1,1)
        self.axes  = self.fig.add_subplot(self.gridspec[0],
                                          facecolor='#FFFFFD')
        self.canvas = FigureCanvasWxAgg(self, -1, self.fig)
        self.canvas.gui_repaint = self.gui_repaint
        self.conf.axes  = self.axes
        self.conf.fig   = self.fig
        self.conf.canvas= self.canvas

        # self.canvas.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))
        # This way of adding to sizer allows resizing
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self.canvas, 1, wx.ALL|wx.GROW)
        self.SetSizer(sizer)
        self.SetSize(self.GetBestVirtualSize())
        self.addCanvasEvents()

    def BuildPopup(self):
        # build pop-up menu for right-click display
        self.popup_menu = popup = wx.Menu()
        MenuItem(self, popup, 'Zoom out', '',   self.unzoom)
        MenuItem(self, popup, 'Zoom all the way out', '',   self.unzoom_all)

        self.popup_menu.AppendSeparator()

        MenuItem(self, popup, 'Rotate 90deg  (CW)', '', self.rotate90)
        MenuItem(self, popup, 'Save Image', '', self.save_figure)

    def rotate90(self, event=None, display=True):
        "rotate 90 degrees, CW"
        self.conf.rotate90()
        if display:
            conf = self.conf
            self.display(conf.data, x=conf.xdata, y=conf.ydata,
                         xlabel=conf.xlab, ylabel=conf.ylab,
                         show_axis=conf.show_axis,
                         levels=conf.ncontour_levels)

    def flip_horiz(self):
        self.conf.flip_horiz()

    def flip_vert(self):
        self.conf.flip_vert()

    def restore_flips_rotations(self):
        "restore flips and rotations"
        conf = self.conf
        if conf.flip_lr:
            self.flip_horiz()
        if conf.flip_ud:
            self.flip_vert()
        if conf.rot_level != 0:
            for i in range(4-conf.rot_level):
                self.rotate90(display=False)
            self.display(conf.data, x=conf.xdata, y=conf.ydata,
                         xlabel=conf.xlab, ylabel=conf.ylab,
                         show_axis=conf.show_axis)




    def toggle_curmode(self, event=None):
        "toggle cursor mode"
        if self.cursor_mode == 'zoom':
            self.cursor_mode = 'lasso'
        else:
            self.cursor_mode = 'zoom'

    ####
    ## GUI events, overriding BasePanel components
    ####
    def calc_indices(self, shape):
        """calculates and stores the set of indices
        ix=[0, nx-1], iy=[0, ny-1] for data of shape (nx, ny)"""
        if len(shape) == 2:
            ny, nx = shape
        elif len(shape) == 3:
            ny, nx, nchan = shape

        inds = []
        for iy in range(ny):
            inds.extend([(ix, iy) for ix in range(nx)])
        self.conf.indices = np.array(inds)

    def lassoHandler(self, vertices):
        if self.conf.indices is None or self.indices_thread.is_alive():
            self.indices_thread.join()
        ind = self.conf.indices
        mask = inside_poly(vertices,ind)
        mask.shape = (self.conf.data.shape[0], self.conf.data.shape[1])
        self.lasso = None
        self.canvas.draw()
        if hasattr(self.lasso_callback , '__call__'):
            self.lasso_callback(mask=mask)

    def unzoom(self, event=None, set_bounds=True):
        """ zoom out 1 level, or to full data range """
        lims = None
        if len(self.conf.zoom_lims) > 1:
            lims = self.conf.zoom_lims.pop()
        ax = self.axes
        if lims is None: # auto scale
            self.conf.zoom_lims = [None]
            xmin, xmax, ymin, ymax = self.data_range
            lims = {self.axes: [xmin, xmax, ymin, ymax]}
        self.set_viewlimits()
        self.canvas.draw()

    def zoom_leftup(self, event=None):
        """leftup event handler for zoom mode  in images"""
        if self.zoom_ini is None:
            return

        ini_x, ini_y, ini_xd, ini_yd = self.zoom_ini
        try:
            dx = abs(ini_x - event.x)
            dy = abs(ini_y - event.y)
        except:
            dx, dy = 0, 0
        t0 = time.time()
        self.rbbox = None
        self.zoom_ini = None
        if (dx > 3) and (dy > 3) and (t0-self.mouse_uptime)>0.1:
            self.mouse_uptime = t0
            zlims, tlims = {}, {}
            ax =  self.axes
            xmin, xmax = ax.get_xlim()
            ymin, ymax = ax.get_ylim()

            zlims[ax] = [xmin, xmax, ymin, ymax]

            if len(self.conf.zoom_lims) == 0:
                self.conf.zoom_lims.append(zlims)


            ax_inv = ax.transData.inverted
            try:
                x1, y1 = ax_inv().transform((event.x, event.y))
            except:
                x1, y1 = self.x_lastmove, self.y_lastmove
            try:
                x0, y0 = ax_inv().transform((ini_x, ini_y))
            except:
                x0, y0 = ini_xd, ini_yd

            tlims[ax] = [int(round(min(x0, x1))), int(round(max(x0, x1))),
                         int(round(min(y0, y1))), int(round(max(y0, y1)))]
            self.conf.zoom_lims.append(tlims)
            # now apply limits:
            self.set_viewlimits()
            if callable(self.zoom_callback):
                self.zoom_callback(wid=self.GetId(), limits=tlims[ax])


    def unzoom_all(self, event=None):
        """ zoom out full data range """
        self.conf.zoom_lims = [None]
        self.unzoom(event)

    def redraw(self, col=0):
        """redraw image, applying
        - log scaling,
        - max/min values from sliders or explicit intensity ranges
        - color map
        - interpolation
        """
        conf = self.conf
        img = conf.data
        if img is None: return
        if len(img.shape) == 2:
            col = 0
        if self.conf.style == 'image':
            if conf.log_scale:
                img = np.log10(1 + 9.0*img)

        # apply intensity scale for current limited (zoomed) image
        if len(img.shape) == 2:
            # apply clipped color scale, as from sliders

            imin = float(conf.int_lo[col])
            imax = float(conf.int_hi[col])
            if conf.log_scale:
                imin = np.log10(1 + 9.0*imin)
                imax = np.log10(1 + 9.0*imax)

            (xmin, xmax, ymin, ymax) = self.conf.datalimits
            if xmin is None:  xmin = 0
            if xmax is None:  xmax = img.shape[1]
            if ymin is None:  ymin = 0
            if ymax is None:  ymax = img.shape[0]


            img = (img - imin)/(imax - imin + 1.e-8)
            mlo = conf.cmap_lo[0]/(1.0*conf.cmap_range)
            mhi = conf.cmap_hi[0]/(1.0*conf.cmap_range)
            if self.conf.style == 'image':
                conf.image.set_data(np.clip((img - mlo)/(mhi - mlo + 1.e-8), 0, 1))
                conf.image.set_interpolation(conf.interp)
        else:
            r, g, b = img[:,:,0], img[:,:,1], img[:,:,2]

            rmin = float(conf.int_lo[0])
            rmax = float(conf.int_hi[0])
            gmin = float(conf.int_lo[1])
            gmax = float(conf.int_hi[1])
            bmin = float(conf.int_lo[2])
            bmax = float(conf.int_hi[2])
            if conf.log_scale:
                rmin = np.log10(1 + 9.0*rmin)
                rmax = np.log10(1 + 9.0*rmax)
                gmin = np.log10(1 + 9.0*gmin)
                gmax = np.log10(1 + 9.0*gmax)
                bmin = np.log10(1 + 9.0*bmin)
                bmax = np.log10(1 + 9.0*bmax)

            rlo = conf.cmap_lo[0]/(1.0*conf.cmap_range)
            rhi = conf.cmap_hi[0]/(1.0*conf.cmap_range)
            glo = conf.cmap_lo[1]/(1.0*conf.cmap_range)
            ghi = conf.cmap_hi[1]/(1.0*conf.cmap_range)
            blo = conf.cmap_lo[2]/(1.0*conf.cmap_range)
            bhi = conf.cmap_hi[2]/(1.0*conf.cmap_range)
            r = (r - rmin)/(rmax - rmin + 1.e-8)
            g = (g - gmin)/(gmax - gmin + 1.e-8)
            b = (b - bmin)/(bmax - bmin + 1.e-8)

            inew = img*1.0
            inew[:,:,0] = np.clip((r - rlo)/(rhi - rlo + 1.e-8), 0, 1)
            inew[:,:,1] = np.clip((g - glo)/(ghi - glo + 1.e-8), 0, 1)
            inew[:,:,2] = np.clip((b - blo)/(bhi - blo + 1.e-8), 0, 1)

            whitebg = conf.tricolor_bg.startswith('wh')

            if whitebg:
                inew = conf.tricolor_white_bg(inew)

            if self.conf.style == 'image':
                conf.image.set_data(inew)
                conf.image.set_interpolation(conf.interp)


        try:
            self.scalebar_rect.remove()
        except:
            pass
        try:
            self.scalebar_text.remove()
        except:
            pass

        if conf.scalebar_show:
            ystep, xstep = conf.scalebar_pixelsize
            if xstep is None or ystep is None:
                ystep, xstep = 1, 1
                if conf.xdata is not None:
                    xstep = abs(np.diff(conf.xdata).mean())
                if conf.ydata is not None:
                    ystep = abs(np.diff(conf.ydata).mean())
                self.scalebar_pixelsize = ystep, xstep
            y, x = conf.scalebar_pos
            y, x = int(y), int(x)
            h, w = conf.scalebar_size
            h, w = int(h), int(w/xstep)
            col =  conf.scalebar_color

            self.scalebar_rect = Rectangle((x, y), w, h,linewidth=1, edgecolor=col,
                                 facecolor=col)
            self.axes.add_patch(self.scalebar_rect)
            if conf.scalebar_showlabel:
                yoff, xoff = conf.scalebar_textoffset
                self.scalebar_text = self.axes.text(x+xoff, y+yoff, conf.scalebar_label,
                                                    color=col)
        self.canvas.draw()
        if callable(self.redraw_callback):
            self.redraw_callback(wid=self.GetId())


    def report_motion(self, event=None):
        if event.inaxes is None:
            return
        fmt = "X,Y= %g, %g"
        x, y  = event.xdata, event.ydata
        if len(self.fig.get_axes()) > 1:
            try:
                x, y = self.axes.transData.inverted().transform((x, y))
            except:
                pass
        if self.motion_sbar is None:
            try:
                self.motion_sbar = self.nstatusbar-1
            except AttributeError:
                self.motion_sbar = 1
        self.write_message(fmt % (x, y), panel=self.motion_sbar)
        conf = self.conf
        if conf.slice_onmotion:
            ix, iy = int(round(x)), int(round(y))
            if (ix >= 0 and ix < conf.data.shape[1] and
                iy >= 0 and iy < conf.data.shape[0]):
                conf.slice_xy = ix, iy
                self.update_slices()

    def report_leftdown(self,event=None):
        if event == None:
            return
        if event.xdata is None or event.ydata is None:
            return

        ix, iy = int(round(event.xdata)), int(round(event.ydata))

        conf = self.conf
        if (ix >= 0 and ix < conf.data.shape[1] and
            iy >= 0 and iy < conf.data.shape[0]):
            pos = ''
            if conf.xdata is not None:
                pos = ' %s=%.4g,' % (conf.xlab, conf.xdata[ix])
            if conf.ydata is not None:
                pos = '%s %s=%.4g,' % (pos, conf.ylab, conf.ydata[iy])
            dval = conf.data[iy, ix]
            if len(conf.data.shape) == 3:
                dval = "%.4g, %.4g, %.4g" % tuple(dval)
            else:
                dval = "%.4g" % dval
            msg = "Pixel [%i, %i], %s Intensity=%s " % (ix, iy, pos, dval)

            self.write_message(msg, panel=0)
            conf.slice_xy = ix, iy
            self.update_slices()
            if hasattr(self.cursor_callback , '__call__'):
                self.cursor_callback(x=event.xdata, y=event.ydata)

    def get_slice_plotframe(self):
        shown = False
        new_plotter = False
        if self.slice_plotframe is not None:
            try:
                self.slice_plotframe.Raise()
                shown = True
            except:
                pass
        if not shown:
            self.slice_plotframe = pf = PlotFrame(self)
            new_plotter = True
            try:
                xpos, ypos = self.parent.GetPosition()
                xsiz, ysiz = self.parent.GetSize()
                pf.SetPosition((xpos+xsiz+10, ypos))
            except:
                pass

        return new_plotter, self.slice_plotframe

    def update_slices(self):
        if self.conf.slices in ('None', None, 0):
            return
        x, y = -1, -1
        try:
            x, y = [int(a) for a in self.conf.slice_xy]
        except:
            return
        if len(self.conf.data.shape) == 3:
            ymax, xmax, nc = self.conf.data.shape
        elif len(self.conf.data.shape) == 2:
            ymax, xmax = self.conf.data.shape
            nc = 0
        else:
            return
        if x < 0 or y < 0 or x > xmax or y > ymax:
            return

        wid = int(self.conf.slice_width)
        new_plotter, pf = self.get_slice_plotframe()

        popts = {'ylabel': 'Intensity', 'linewidth': 3}

        if self.conf.slices.lower() == 'x':
            y1 = int(y - wid/2. + 1)
            y2 = int(y + wid/2.) + 1
            if y1 < 0: y1 = 0
            if y2 > ymax: y2 = ymax
            _x = self.conf.xdata
            if _x is None:
                _x = np.arange(self.conf.data.shape[1])
            _y = self.conf.data[y1:y2].sum(axis=0)
            popts['xlabel'] = 'X'
            popts['title'] = 'X Slice: Y=%d:%d' % (y1, y2)
            if y2 == y1+1:
                popts['title'] = 'X Slice: Y=%d' % y1

        else:
            x1 = int(x - wid/2.0 + 1)
            x2 = int(x + wid/2.0) + 1
            if x1 < 0: x1 = 0
            if x2 > xmax: x2 = xmax
            _x = self.conf.ydata
            if _x is None:
                _x = np.arange(self.conf.data.shape[0])
            _y = self.conf.data[:,x1:x2].sum(axis=1)
            popts['xlabel'] = 'Y'
            popts['title'] = 'Y Slice: X=%d:%d' % (x1, x2)
            if x2 == x1+1:
                popts['title'] = 'Y Slice: X=%d' % x1

        if new_plotter:
            if len(_y.shape) == 2 and _y.shape[1] == 3:
                pf.plot(_x,  _y[:, 0], color=RGB_COLORS[0], delay_draw=True, **popts)
                pf.oplot(_x, _y[:, 1], color=RGB_COLORS[1], delay_draw=True, **popts)
                pf.oplot(_x, _y[:, 2], color=RGB_COLORS[2], **popts)
            else:
                pf.plot(_x, _y, **popts)
        else:
            pf.panel.set_title(popts['title'], delay_draw=True)
            pf.panel.set_xlabel(popts['xlabel'], delay_draw=True)
            if len(_y.shape) == 2 and _y.shape[1] == 3:
                pf.update_line(0, _x,  _y[:, 0], update_limits=True, draw=False)
                pf.update_line(1, _x,  _y[:, 1], update_limits=True, draw=False)
                pf.update_line(2, _x,  _y[:, 2], update_limits=True, draw=True)
            else:
                pf.update_line(0, _x, _y, update_limits=True, draw=True)


        pf.Show()
        self.SetFocus()
        try:
            self.parent.Raise()
        except:
            pass



#!/usr/bin/python
from os import path
import numpy as np
import wxmplot.interactive as wi

fname = 'xafs.dat'
thisdir, _ = path.split(__file__)
dat = np.loadtxt(path.join(thisdir, fname))

x = dat[:, 0]
y = dat[:, 1]

wi.plot(x, y, xlabel='E (eV)', ylabel=r'$\mu(E)$',
        label='As K edge', title='Data from %s' % fname)
